/*
 * event.h - Public Event header
 *
 * Should be included anywhere needed: netinfo, netinfod, modules, etc
 *
 *  (c)2003,2011 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _EVENT_H_
#define _EVENT_H_


#define	EVENT_FLAG_ONESHOT		0x00000001
#define EVENT_FLAG_HEARTBEAT		0x00000002

#define EC_OPTION_HEARBEAT_INTERVAL	1
#define EC_OPTION_HEARBEAT_OBJECTNAME	2
#define EC_OPTION_LOGGING_FUNCTION	3
#define EC_OPTION_ENABLE_DEBUG		4

#define EC_FLAG_DEBUG			0x00000001


typedef struct _EventData		EventData;
typedef struct _EventChain		EventChain;
typedef struct _EventClient		EventClient;

NETINFO_API int	eventdata_new (EventData **edp);
NETINFO_API int	eventdata_free (EventData **edp);
NETINFO_API int	eventdata_clear (EventData *ed);
NETINFO_API int	eventdata_set (EventData *ed,
			u_int32_t flags,
			u_int32_t rc,
			char *object,
			char *data,
			u_int32_t datalen);
NETINFO_API int	eventdata_set_all (EventData *ed,
			char *eventhostname,
			u_int32_t flags,
			u_int32_t rc,
			char *object,
			char *data,
			u_int32_t datalen);
NETINFO_API int	eventdata_to_hostorder (EventData *ed);
NETINFO_API int	eventdata_to_networkorder (EventData *ed);
NETINFO_API int	eventdata_get_hostname (EventData *ed, char *hostname, int *len);
NETINFO_API int	eventdata_get_object (EventData *ed, char *object, int *len);
NETINFO_API int	eventdata_get_data (EventData *ed, char *data, int *len);
NETINFO_API int	eventdata_get_flags (EventData *ed, u_int32_t *flags);
NETINFO_API int	eventdata_get_rc (EventData *ed, u_int32_t *rc);
NETINFO_API int	eventdata_get_timestamp (EventData *ed, u_int64_t *timestamp);
NETINFO_API int	eventdata_sizeof (EventData *ed, int *size);

NETINFO_API int	eventchain_open (EventChain **ecp);
NETINFO_API int	eventchain_close (EventChain ** ecp);
NETINFO_API int	eventchain_emit_event (EventChain *ec, EventData *ed);

NETINFO_API int	eventclient_init (EventClient **ecp, u_int32_t flags);
NETINFO_API int	eventclient_uninit (EventClient **ecp);
NETINFO_API int	eventclient_emit_event (EventClient *ec, EventData *ed);
NETINFO_API int	eventclient_configure_heartbeat (EventClient *ec,
					char *object,
					int interval);
NETINFO_API int eventclient_set_option (EventClient *ec,
					u_int32_t option,
					void *valuep);

#endif

