/*
 * event_p.h - Private Event header
 *
 *  (c)2003 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _EVENT_P_H_
#define _EVENT_P_H_

#include <sys/types.h>

#include <cutil/cutil.h>
#include <comm/comm.h>

#include <netinfo/netinfo_portable.h>


#define	EVENT_HOSTNAME_SIZE		64
#define	EVENT_OBJECT_SIZE		64
#define	EVENT_DATA_SIZE			256


#define	EVENTTHREAD_FLAG_CONNECTED	1
#define	EVENTTHREAD_FLAG_EXITTHREAD	2
#define EVENTTHREAD_HEARTBEAT_OBJECT	"HEARTBEAT"
#define	EVENTTHREAD_HEARTBEAT_INTERVAL	300
#define EVENTCLIENT_EXIT_WAIT_INTERVAL	5
#define EVENTCLIENT_QUEUE_SIZE		100

struct _EventData {
	u_int32_t		flags;
	u_int32_t		rc;
	u_int64_t		timestamp;
	u_int32_t		datalen;
	char			hostname[EVENT_HOSTNAME_SIZE];
	char			object[EVENT_OBJECT_SIZE];
	char			data[EVENT_DATA_SIZE];
#ifndef WIN32
	u_int32_t		pad1;
#endif
};

struct _EventChain {
	void		*commchannel;
	void		*moduleinfo;
	unsigned long	eventcount;
};

typedef int (*EventClientLogFunction)(char *format, ...);

struct _EventClient {
	CommModule	*commmodule;
	CommChannel	*commchannel;
	Attribute	*attributelist;
	char		eventhost[EVENT_HOSTNAME_SIZE];
	int		eventport;
	char		*heartbeatobject;
	int		heartbeatinterval;
	u_int32_t	threadflags;
	pthread_t	thread;
	pthread_attr_t	threadattr;
	pthread_mutex_t	queuemutex;
	pthread_cond_t	threadcond;
	SQueue		*eventqueue;
	u_int32_t	clientflags;
	EventClientLogFunction	logfunction;
	
};

#endif

