/****************************************************************/
/* netinfo.h - netinfo client/server common definitions		*/
/****************************************************************/
/* History:							*/
/* (02-10-2005) v0.8.3 Added protocol v2/module stuff. Todd	*/
/* (12-16-2001) v0.8.1 Added protocol negotiation code. Todd	*/
/****************************************************************/
/*
 *  (c)2003 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef NETINFO_H_
#define NETINFO_H_

#include <sys/types.h>

#include <netinfo/netinfo_portable.h>

#define NETINFO_DEFAULT_PORT		5010
#define	NIEVENT_DEFAULT_PORT		5011

/* v2 stuff */
#define NETINFO_PROTO_VERSION_2		2

#define NETINFO_V2_COMMAND_SIZE		32

#define	NETINFO_FLAG_DATAPRESENT	0x0001

#define NETINFO_STATUS_REQUEST		0x01000000
#define NETINFO_STATUS_RESPONSE		0x02000000
#define NETINFO_STATUS_REQERROR		0x00000001
#define NETINFO_STATUS_CMDINVALID	0x00000002
#define NETINFO_STATUS_ARGINVALID	0x00000004
#define NETINFO_STATUS_SERVERERROR	0x00000008


/* Return Codes for the various internal functions */
#define	NETINFO_RC_SUCCESS		0
#define	NETINFO_RC_INVALIDARG		1
#define	NETINFO_RC_MEMORYERROR		2
#define	NETINFO_RC_SSLERROR		3
#define	NETINFO_RC_LOADERROR		4
#define	NETINFO_RC_MODULEERROR		5	
#define	NETINFO_RC_NOTIMPLEMENTED	6
#define	NETINFO_RC_ENDOFFILE		7
#define	NETINFO_RC_PROTOCOLERROR	8


#define NETINFO_HEADER_COMMAND_SIZE	32
#define NETINFO_HEADER_ARGDATA_SIZE	128


/*
#ifdef WIN32
#include <windows.h>
#define        PTHREAD_CREATE_DETACHED 1
#define        ENODATA                 61
typedef        unsigned char           u_int8_t;
typedef        WORD                    u_int16_t;
typedef        DWORD                   u_int32_t;
typedef        unsigned long long      u_int64_t;
typedef        DWORD                   pthread_mutex_t;
typedef        DWORD                   pthread_attr_t;
typedef        DWORD                   pthread_mutexattr_t;
typedef        DWORD                   pthread_condattr_t;
typedef        DWORD                   pthread_t;
typedef        DWORD                   pthread_cond_t;
#define        pthread_mutex_create()  CreateMutex (NULL, FALSE, NULL)
#define        pthread_mutex_lock(x)   WaitForSingleObject(x, INFINITE)
#define        pthread_mutex_unlock(x) ReleaseMutex(x)
#define        pthread_exit(x)         ExitThread((DWORD)x)
#define        pthread_attr_init(x)
#define        pthread_attr_setdetachstate(x,y)
int pthread_mutex_init (pthread_mutex_t *mutex,
			pthread_mutexattr_t *mutexattr);
int pthread_create(pthread_t  *thread,
			pthread_attr_t *attr,
			void * (*start_routine)(void *),
			void *arg);
int pthread_cond_init(pthread_cond_t *cond,
			pthread_condattr_t *cond_attr);
#endif
*/

typedef struct _NetinfoHeader	NetinfoHeader;
struct _NetinfoHeader {
	u_int32_t	status;
	u_int32_t	flags;
	u_int32_t	datalen;
	u_int32_t	rc;
	u_int32_t	arglen;
	char	command[NETINFO_HEADER_COMMAND_SIZE];
	char	argdata[NETINFO_HEADER_ARGDATA_SIZE];
};

NETINFO_API int netinfo_header_clear (NetinfoHeader *nh);
NETINFO_API int netinfo_header_set_flags (NetinfoHeader *nh, u_int32_t flags);
NETINFO_API int netinfo_header_get_flags (NetinfoHeader *nh, u_int32_t *pflags);
NETINFO_API int netinfo_header_set_status (NetinfoHeader *nh, u_int32_t status);
NETINFO_API int netinfo_header_get_status (NetinfoHeader *nh, u_int32_t *pstatus);
NETINFO_API int netinfo_header_set_datalen (NetinfoHeader *nh, u_int32_t datalen);
NETINFO_API int netinfo_header_get_datalen (NetinfoHeader *nh, u_int32_t *pdatalen);
NETINFO_API int netinfo_header_set_rc (NetinfoHeader *nh, u_int32_t rc);
NETINFO_API int netinfo_header_get_rc (NetinfoHeader *nh, u_int32_t *prc);
NETINFO_API int netinfo_header_set_command (NetinfoHeader *nh, char *command);
NETINFO_API int netinfo_header_get_command (NetinfoHeader *nh, char *command, int len);
NETINFO_API int netinfo_header_set_argdata (NetinfoHeader *nh, char *argdata, int len);
NETINFO_API int netinfo_header_get_argdata (NetinfoHeader *nh, char *argdata, int *len);
NETINFO_API int netinfo_header_to_networkorder (NetinfoHeader *nh);
NETINFO_API int netinfo_header_to_hostorder (NetinfoHeader *nh);
/* v2 stuff */

#endif

