/*
 * nmodule_p.h - Private NModule header
*/
/*
 *  (c)2003 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef _NMODULE_P_H_
#define _NMODULE_P_H_

#ifndef WIN32
#include <dlfcn.h>
#define	MODULEHANDLE		void*
#else
#include <winbase.h>
#define	MODULEHANDLE		HMODULE
#endif

#include <stdarg.h>

#include <netinfo/netinfo.h>
#include <netinfo/event.h>

#define	MODULE_VECTOR_VERSION_1		1
#define	MODULE_VECTOR_VERSION		MODULE_VECTOR_VERSION_1


typedef struct _NetinfoModule		NetinfoModule;
typedef struct _ModuleVector		ModuleVector;
typedef struct _ModuleInfo		ModuleInfo;

typedef int (*ModuleInitFunc)(ModuleInfo *mi);
typedef int (*ModuleCallMethodFunc)(char *method,
				NetinfoHeader *nh,
				char **responsedata,
				u_int32_t *responsedatalen);
typedef int (*ModuleCloseFunc)(ModuleInfo *mi);

struct _ModuleVector {
	unsigned long	version;
	int		(*log_entry)(ModuleInfo *mi,
					char *format,
					va_list args);
	int		(*emit_event)(ModuleInfo *mi, EventData *ed);
	int		(*eventchain_open)(ModuleInfo *mi, EventChain **ec);
	int		(*eventchain_close)(EventChain **ec);
	int		(*eventchain_emit_event)(EventChain *ec, EventData *ed);
};

struct _ModuleInfo {
	const char		*name;	/* set by module */
	ModuleVector		*vector; /* set by netinfod */
	void			*data;	/* set by module (optional) */
};

struct _NetinfoModule {
	MODULEHANDLE		module;
	ModuleInfo		moduleinfo;
	ModuleInitFunc		moduleinit;
	ModuleCallMethodFunc	modulecallmethod;
	ModuleCloseFunc		moduleclose;
	unsigned long		flags;
	unsigned long		refcount;
};


/* Functions for netinfod use */
int	netinfo_vector_initialize (ModuleVector *mv);
int	netinfo_vector_set_log_entry (ModuleVector *mv, void *callback);
int	netinfo_vector_set_emit_event (ModuleVector *mv, void *callback);
int	netinfo_module_load (NetinfoModule **nmp, char *path, ModuleVector *mv);
const char*	netinfo_module_get_name (NetinfoModule *nm);
int	netinfo_module_call_method (NetinfoModule *nm,
					char *method,
					NetinfoHeader *nhresponse,
					char **responsedata,
					u_int32_t *responselen);
int	netinfo_module_unload (NetinfoModule **nmp);

#endif

