/************************************************************************/
/* rpc.h - Netinfo RPC Header						*/
/************************************************************************/
/*
 *  (c)2003 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef _NETINFO_RPC_H_
#define _NETINFO_RPC_H_

#include <netinfo/netinfo.h>

#define	NETINFO_RPC_CALLID_FILEOPEN	0x01001001
#define	NETINFO_RPC_CALLID_FILECLOSE	0x01001002
#define	NETINFO_RPC_CALLID_FILEDATA	0x01001003
#define	NETINFO_RPC_CALLID_RESTART	0x01001004
/* RPC version# is the high 16 bits of the CALLID */
#define NETINFO_GET_VERSION_MAJOR(x)	(x & 0xff000000)
#define NETINFO_GET_VERSION_MINOR(x)	(x & 0x00ff0000)
#define NETINFO_GET_CALLID(x)		(x & 0x0000ffff)


/* RPC returncode = errno */

/* Used in 'access' and 'direction' fields */
#define	NETINFO_RPC_READ		0x00000001
#define	NETINFO_RPC_WRITE		0x00000002
/* Used in 'access' fields */
#define	NETINFO_RPC_READWRITE		0x00000003
#define	NETINFO_RPC_DELETE		0x00000004
#define	NETINFO_RPC_EXECPIPE		0x00000005

#define	NETINFO_RPC_FILEPATH_SIZE	1024


typedef struct _NetinfoRpcHeader	NetinfoRpcHeader;
struct _NetinfoRpcHeader {
	u_int32_t	callid;
	u_int32_t	requestid;
	u_int32_t	returncode;
	u_int32_t	reserved1;
};

typedef struct _NetinfoRpcFileOpen	NetinfoRpcFileOpen;
struct _NetinfoRpcFileOpen {
	u_int32_t	fileid;
	u_int32_t	access;
	u_int32_t	reserved1;
	unsigned char	filepath[NETINFO_RPC_FILEPATH_SIZE];
};

typedef struct _NetinfoRpcFileClose	NetinfoRpcFileClose;
struct _NetinfoRpcFileClose {
	u_int32_t	fileid;
	u_int32_t	reserved1;
};

typedef struct _NetinfoRpcFileData	NetinfoRpcFileData;
struct _NetinfoRpcFileData {
	u_int32_t	fileid;
	u_int32_t	direction;
	u_int64_t	offset;
	u_int32_t	length;
	/* file data immediately following this structure */
};

typedef struct _NetinfoRpcListHead	NetinfoRpcListHead;
struct _NetinfoRpcListHead {
	u_int32_t	itemsize;
	u_int32_t	itemcount;
};


NETINFO_API void netinfo_rpcheader_hton(NetinfoRpcHeader *rpcheader);
NETINFO_API void netinfo_rpcheader_ntoh(NetinfoRpcHeader *rpcheader);
NETINFO_API void netinfo_rpcfileopen_hton(NetinfoRpcFileOpen *rpcfileopen);
NETINFO_API void netinfo_rpcfileopen_ntoh(NetinfoRpcFileOpen *rpcfileopen);
NETINFO_API void netinfo_rpcfileclose_hton(NetinfoRpcFileClose *rpcfileclose);
NETINFO_API void netinfo_rpcfileclose_ntoh(NetinfoRpcFileClose *rpcfileclose);
NETINFO_API void netinfo_rpcfiledata_hton(NetinfoRpcFileData *rpcfiledata);
NETINFO_API void netinfo_rpcfiledata_ntoh(NetinfoRpcFileData *rpcfiledata);
NETINFO_API void netinfo_rpclisthead_hton(NetinfoRpcListHead *rpclisthead);
NETINFO_API void netinfo_rpclisthead_ntoh(NetinfoRpcListHead *rpclisthead);

#endif


