/************************************************************************/
/*
 * conffile.c- Configuration admin source
 *
 * (c)2005) Todd Shadburn
 *
*/
/************************************************************************/


#ifdef WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <netinfo/netinfo.h>
#include <netinfo/conffile.h>
#include <netinfo/conffile_p.h>

NETINFO_API int configfile_open (ConfigFile **cfp,
			char *path)
{
	ConfigFile *cf = NULL;
	if (cfp == NULL || path == NULL)
		return NETINFO_RC_INVALIDARG;
	*cfp = NULL;
	cf = calloc (1, sizeof (ConfigFile));
	if (cf == NULL)
		return NETINFO_RC_MEMORYERROR;
	cf->fd = fopen (path, "r");
	if (cf->fd == NULL) {
		free (cf);
		return NETINFO_RC_INVALIDARG;
	}
	cf->readoffset = ftell (cf->fd);
	cf->searchoffset = ftell (cf->fd);
	cf->path = strdup (path);
	*cfp = cf;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int configfile_read_line (ConfigFile *cf,
			char *buffer,
			int len)
{
	if (cf == NULL || buffer == NULL)
		return NETINFO_RC_INVALIDARG;
	if (feof(cf->fd))
		return NETINFO_RC_ENDOFFILE;
	fseek (cf->fd, cf->readoffset, SEEK_SET);
	fgets (buffer, len, cf->fd);
	cf->readoffset = ftell (cf->fd);
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int configfile_append_line (ConfigFile *cf,
			char *buffer,
			int len)
{
	if (cf == NULL || buffer == NULL)
		return NETINFO_RC_INVALIDARG;
	fseek (cf->fd, 0, SEEK_END);
	fwrite (buffer, 1, len, cf->fd);
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int configfile_search_begin (ConfigFile *cf,
			char *searchstr)
{
	if (cf == NULL || searchstr == NULL)
		return NETINFO_RC_INVALIDARG;
	cf->searchstr = strdup (searchstr);
	cf->searchoffset = 0;
	fseek (cf->fd, cf->searchoffset, SEEK_SET);
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int configfile_search_next (ConfigFile *cf,
			char *buffer,
			int len)
{
	if (cf == NULL || buffer == NULL || cf->searchstr == NULL)
		return NETINFO_RC_INVALIDARG;
	if (len < 1)
		return NETINFO_RC_INVALIDARG;
	fseek (cf->fd, cf->searchoffset, SEEK_SET);
	while (!feof (cf->fd)) {
		if (fgets (buffer, len, cf->fd) == NULL) {
			buffer[0] = 0;
			return NETINFO_RC_ENDOFFILE;
		}
		cf->searchoffset = ftell (cf->fd);
		if (strstr (buffer, cf->searchstr) != NULL)
			return NETINFO_RC_SUCCESS;
	}
	buffer[0] = 0;
	return NETINFO_RC_ENDOFFILE;
}


NETINFO_API int configfile_search_end (ConfigFile *cf)
{
	if (cf == NULL)
		return NETINFO_RC_INVALIDARG;
	if (cf->searchstr != NULL) {
		free (cf->searchstr);
	}
	cf->searchstr = NULL;
	cf->searchoffset = 0;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int configfile_close (ConfigFile **cfp)
{
	ConfigFile *cf = NULL;
	if (cfp == NULL || *cfp == NULL)
		return NETINFO_RC_INVALIDARG;
	cf = *cfp;
	if (cf->fd != NULL) {
		fflush (cf->fd);
		fclose (cf->fd);
	}
	if (cf->path != NULL)
		free (cf->path);
	if (cf->searchstr != NULL)
		free (cf->searchstr);
	free (cf);
	*cfp = NULL;
	return NETINFO_RC_SUCCESS;
}




