/*
 * event.c - Event source
 *
*/

/*
#ifndef WIN32
#include <netinet/in.h>
#include <unistd.h>
#else
#include <windows.h>
#include <winsock2.h>
#endif
*/

#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <errno.h>

#include <cutil/cutil.h>
#include <comm/comm.h>

#include <netinfo/netinfo.h>
#include <netinfo/conffile.h>
#include <netinfo/event.h>
#include <netinfo/event_p.h>


#define	STRING_SIZE	256


/* Function Prototypes - Internal */
void	*eventclient_thread (void *arg);
int     eventclient_prepare_and_write_event (EventClient *ec, EventData *ed);
int	eventclient_test_and_connect (EventClient *ec);


NETINFO_API int     eventdata_new (EventData **edp)
{
	EventData *ed = NULL;
	if (edp == NULL)
		return NETINFO_RC_INVALIDARG;
	*edp = NULL;
	ed = calloc (1, sizeof (EventData));
	if (ed == NULL)
		return NETINFO_RC_MEMORYERROR;
	*edp = ed;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int     eventdata_free (EventData **edp)
{
	EventData *ed = NULL;
	if (edp == NULL || *edp == NULL)
		return NETINFO_RC_INVALIDARG;
	ed = *edp;
	free (ed);
	*edp = NULL;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int	eventdata_clear (EventData *ed)
{
	if (ed == NULL)
		return NETINFO_RC_INVALIDARG;
	memset (ed, 0, sizeof (EventData));
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int	eventdata_set (EventData *ed,
			u_int32_t eventflags,
			u_int32_t eventrc,
			char *eventobject,
			char *eventdata,
			u_int32_t eventdatalen)
{
	int rc;
	rc = eventdata_set_all (ed,
			NULL,
			eventflags,
			eventrc,
			eventobject,
			eventdata,
			eventdatalen);
	return rc;
}


NETINFO_API int	eventdata_set_all (EventData *ed,
			char *eventhostname,
			u_int32_t eventflags,
			u_int32_t eventrc,
			char *eventobject,
			char *eventdata,
			u_int32_t eventdatalen)
{
	int rc;
	time_t t;
	if (ed == NULL || eventobject == NULL)
		return NETINFO_RC_INVALIDARG;
	rc = eventdata_clear (ed);
	t = time (NULL);
	ed->timestamp = t;
	if (eventhostname == NULL)
		rc = gethostname (ed->hostname, EVENT_HOSTNAME_SIZE-1);	
	else
		strncpy (ed->hostname, eventhostname, EVENT_HOSTNAME_SIZE-1);
	ed->flags = eventflags;
	ed->rc = eventrc;
	strncpy (ed->object, eventobject, EVENT_OBJECT_SIZE-1);
	strncpy (ed->data, eventdata, EVENT_DATA_SIZE);
	if (eventdatalen <= EVENT_DATA_SIZE)
		ed->datalen = eventdatalen;
	else
		ed->datalen = EVENT_DATA_SIZE;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int	eventdata_to_hostorder (EventData *ed)
{
	if (ed == NULL)
		return NETINFO_RC_INVALIDARG;
	ed->timestamp = ntohl (ed->timestamp);
	ed->flags = ntohl (ed->flags);
	ed->rc = ntohl (ed->rc);
	ed->datalen = ntohl (ed->datalen);
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int	eventdata_to_networkorder (EventData *ed)
{
	if (ed == NULL)
		return NETINFO_RC_INVALIDARG;
	ed->timestamp = htonl (ed->timestamp);
	ed->flags = htonl (ed->flags);
	ed->rc = htonl (ed->rc);
	ed->datalen = htonl (ed->datalen);
	return NETINFO_RC_SUCCESS;
}

NETINFO_API int     eventdata_get_hostname (EventData *ed, char *hostname, int *len)
{
	int size = EVENT_HOSTNAME_SIZE;
	if (ed == NULL || hostname == NULL || len == NULL)
		return NETINFO_RC_INVALIDARG;
	if (*len < size)
		size = *len;
	strncpy (hostname, ed->hostname, size-1);
	*len = strlen (ed->hostname);
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int     eventdata_get_object (EventData *ed, char *object, int *len)
{
	int size = EVENT_OBJECT_SIZE;
	if (ed == NULL || object == NULL || len == NULL)
		return NETINFO_RC_INVALIDARG;
	if (*len < size)
		size = *len;
	strncpy (object, ed->object, size-1);
	*len = strlen (ed->object);
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int     eventdata_get_data (EventData *ed, char *data, int *len)
{
	int size;
	if (ed == NULL || data == NULL || len == NULL)
		return NETINFO_RC_INVALIDARG;
	size = ed->datalen;
	if (*len < size)
		size = *len;
	strncpy (data, ed->data, size);
	*len = size;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int     eventdata_get_flags (EventData *ed, u_int32_t *flags)
{
	if (ed == NULL || flags == NULL)
		return NETINFO_RC_INVALIDARG;
	*flags = ed->flags;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int     eventdata_get_rc (EventData *ed, u_int32_t *rc)
{
	if (ed == NULL || rc == NULL)
		return NETINFO_RC_INVALIDARG;
	*rc = ed->rc;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int     eventdata_get_timestamp (EventData *ed, u_int64_t *timestamp)
{
	if (ed == NULL || timestamp == NULL)
		return NETINFO_RC_INVALIDARG;
	*timestamp = ed->timestamp;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int     eventdata_sizeof (EventData *ed, int *size)
{
	if (ed == NULL) {
		*size = 0L;
		return NETINFO_RC_INVALIDARG;
	}
	*size = sizeof (EventData);
	return NETINFO_RC_SUCCESS;
}

/*
int     eventchain_open (EventChain **ecp)
{
	int             rc;
	int		eventport;
	char		configline[STRING_SIZE];
	char		eventhost[STRING_SIZE];
	char		*privatekey = NULL;
	char		*publickey = NULL;
	char		*directive = NULL;
	ConfigFile	*cf = NULL;
	CommChannel     *cc = NULL;
	EventChain      *ec = NULL;
	Attribute	*attribute = NULL;
	if (ecp == NULL)
		return NETINFO_RC_INVALIDARG;
	*ecp = NULL;
	eventport = NIEVENT_DEFAULT_PORT;
	memset (eventhost, 0, STRING_SIZE);
	rc = configfile_open (&cf, SYSCONFDIR "/netinfod.conf");
	if (rc != NETINFO_RC_SUCCESS)
		return rc;
	rc = configfile_search_begin (cf, "PRIVATEKEY");
	rc = configfile_search_next (cf, configline, STRING_SIZE);
	if (rc != NETINFO_RC_SUCCESS) {
		configfile_search_end (cf);
		configfile_close (&cf);
		return rc;
	}
	rc = configfile_search_end (cf);
	directive = strtok (configline, " \t\r\n");
	privatekey = strdup (strtok (NULL, "\r\n"));
	attribute = attribute_add (attribute,
				"SSLPrivateKeyFile",
				privatekey);
	rc = configfile_search_begin (cf, "PUBLICKEY");
	rc = configfile_search_next (cf, configline, STRING_SIZE);
	if (rc != NETINFO_RC_SUCCESS) {
		configfile_search_end (cf);
		configfile_close (&cf);
		attribute_freeall (&attribute);
		free (privatekey);
		return rc;
	}
	rc = configfile_search_end (cf);
	directive = strtok (configline, " \t\r\n");
	publickey = strdup (strtok (NULL, "\r\n"));
	attribute = attribute_add (attribute,
				"SSLCertificateFile",
				publickey);
	rc = configfile_search_begin (cf, "EVENTHOST");
	rc = configfile_search_next (cf, configline, STRING_SIZE);
	if (rc != NETINFO_RC_SUCCESS) {
		configfile_search_end (cf);
		configfile_close (&cf);
		attribute_freeall (&attribute);
		return rc;
	}
	directive = strtok (configline, " \t\r\n");
	directive = strtok (NULL, "\r\n");
	strncpy (eventhost, directive, STRING_SIZE);
	rc = configfile_search_end (cf);
	rc = configfile_search_begin (cf, "EVENTPORT");
	rc = configfile_search_next (cf, configline, STRING_SIZE);
	if (rc == NETINFO_RC_SUCCESS) {
		directive = strtok (configline, " \t\r\n");
		directive = strtok (NULL, " \t\r\n");
		eventport = atoi (directive);
	}
	rc = configfile_search_end (cf);
	rc = configfile_close (&cf);
	ec = calloc (1, sizeof (EventChain));
	if (ec == NULL) {
		attribute_freeall (&attribute);
		return NETINFO_RC_MEMORYERROR;
	}
	cc = calloc (1, sizeof (CommChannel));
	if (cc == NULL) {
		attribute_freeall (&attribute);
		free (ec);
		return NETINFO_RC_MEMORYERROR;
	}
	cc->cmodule = calloc (1, sizeof (CommModule));
	if (cc->cmodule == NULL) {
		attribute_freeall (&attribute);
		free (cc);
		free (ec);
		return NETINFO_RC_MEMORYERROR;
	}
	cc->cmodule->moddata = calloc (1, sizeof (CommModuleData));
	if (cc->cmodule == NULL) {
		attribute_freeall (&attribute);
		free (cc->cmodule);
		free (cc);
		free (ec);
		return NETINFO_RC_MEMORYERROR;
	}
	cc->cmodule->moddata->attribute = attribute;
	cc->cmodule->moddata->vector = calloc (1, sizeof (CommVectors));
	if (cc->cmodule->moddata->vector == NULL) {
		attribute_freeall (&attribute);
		free (cc->cmodule->moddata);
		free (cc->cmodule);
		free (cc);
		free (ec);
		return NETINFO_RC_MEMORYERROR;
	}
	rc = openssl_comm_init (cc->cmodule->moddata);
	if (rc != 0) {
		attribute_freeall (&attribute);
		free (cc->cmodule->moddata->vector);
		free (cc->cmodule->moddata);
		free (cc->cmodule);
		free (cc);
		free (ec);
		return rc;
	}
	rc = openssl_comm_connect (cc,
				eventhost,
				eventport);
	if (rc != 0) {
		attribute_freeall (&attribute);
		free (cc->cmodule->moddata->vector);
		free (cc->cmodule->moddata);
		free (cc->cmodule);
		free (cc);
		free (ec);
		return rc;
	}
	ec->commchannel = cc;
	*ecp = ec;
	return NETINFO_RC_SUCCESS;
}


int     eventchain_close (EventChain **ecp)
{
	int		rc;
	EventChain	*ec = NULL;
	CommChannel     *cc = NULL;
	if (ecp == NULL || *ecp == NULL)
		return NETINFO_RC_INVALIDARG;
	ec = *ecp;
	if (ec->commchannel != NULL) {
		cc = ec->commchannel;
		rc = openssl_comm_close (cc, 0);
		attribute_freeall (&cc->cmodule->moddata->attribute);
		free (cc->cmodule->moddata->vector);
		free (cc->cmodule->moddata);
		free (cc->cmodule);
		free (cc);
	}
	free (ec);
	*ecp = NULL;
	return NETINFO_RC_SUCCESS;
}


int     eventchain_emit_event (EventChain *ec, EventData *ed)
{
	int             rc;
	int             bytes;
	CommChannel     *cc = NULL;
	if (ec == NULL || ed == NULL)
		return NETINFO_RC_INVALIDARG;
	if (ec->commchannel == NULL)
		return NETINFO_RC_INVALIDARG;
	cc = ec->commchannel;
	rc = eventdata_to_networkorder (ed);
	rc = eventdata_sizeof (ed, &bytes);
	rc = openssl_comm_write (cc, (char *)ed, &bytes);
	return rc;
}
*/

NETINFO_API int     eventclient_init (EventClient **ecp, u_int32_t flags)
{
	int             rc;
	char		configline[STRING_SIZE];
	char		*privatekey = NULL;
	char		*publickey = NULL;
	char		*directive = NULL;
	ConfigFile	*cf = NULL;
	EventClient      *ec = NULL;
	if (ecp == NULL)
		return NETINFO_RC_INVALIDARG;
	*ecp = NULL;
	ec = calloc (1, sizeof (EventClient));
	if (ec == NULL) {
		return NETINFO_RC_MEMORYERROR;
	}
	ec->eventport = NIEVENT_DEFAULT_PORT;
	memset (ec->eventhost, 0, EVENT_HOSTNAME_SIZE);
	rc = configfile_open (&cf, SYSCONFDIR "/netinfod.conf");
	if (rc != NETINFO_RC_SUCCESS)
		return rc;
	rc = configfile_search_begin (cf, "PRIVATEKEY");
	rc = configfile_search_next (cf, configline, STRING_SIZE);
	if (rc != NETINFO_RC_SUCCESS) {
		configfile_search_end (cf);
		configfile_close (&cf);
		return rc;
	}
	rc = configfile_search_end (cf);
	directive = strtok (configline, " \t\r\n");
	privatekey = strdup (strtok (NULL, "\r\n"));
	ec->attributelist = attribute_add (ec->attributelist,
				"SSLPrivateKeyFile",
				privatekey);
	rc = configfile_search_begin (cf, "PUBLICKEY");
	rc = configfile_search_next (cf, configline, STRING_SIZE);
	if (rc != NETINFO_RC_SUCCESS) {
		configfile_search_end (cf);
		configfile_close (&cf);
		attribute_freeall (&ec->attributelist);
		free (ec);
		free (privatekey);
		return rc;
	}
	rc = configfile_search_end (cf);
	directive = strtok (configline, " \t\r\n");
	publickey = strdup (strtok (NULL, "\r\n"));
	ec->attributelist = attribute_add (ec->attributelist,
				"SSLCertificateFile",
				publickey);
	rc = configfile_search_begin (cf, "EVENTHOST");
	rc = configfile_search_next (cf, configline, STRING_SIZE);
	if (rc != NETINFO_RC_SUCCESS) {
		configfile_search_end (cf);
		configfile_close (&cf);
		attribute_freeall (&ec->attributelist);
		free (ec);
		return rc;
	}
	directive = strtok (configline, " \t\r\n");
	directive = strtok (NULL, "\r\n");
	strncpy (ec->eventhost, directive, EVENT_HOSTNAME_SIZE);
	rc = configfile_search_end (cf);
	rc = configfile_search_begin (cf, "EVENTPORT");
	rc = configfile_search_next (cf, configline, STRING_SIZE);
	if (rc == NETINFO_RC_SUCCESS) {
		directive = strtok (configline, " \t\r\n");
		directive = strtok (NULL, " \t\r\n");
		ec->eventport = atoi (directive);
	}
	rc = configfile_search_end (cf);
	rc = configfile_close (&cf);
	rc = comm_plugin_load (&ec->commmodule,
			"comm_openssl",
			ec->attributelist);
	if (rc != 0) {
		attribute_freeall (&ec->attributelist);
		free (ec);
		return rc;
	}
	rc = squeue_new (&ec->eventqueue,
			sizeof (EventData),
			EVENTCLIENT_QUEUE_SIZE);
	if (rc != 0) {
		comm_plugin_unload (&ec->commmodule);
		attribute_freeall (&ec->attributelist);
		free (ec);
		return rc;
	}
	rc = pthread_mutex_init (&ec->queuemutex, NULL);
	rc = pthread_cond_init (&ec->threadcond, NULL);
	rc = pthread_attr_init (&ec->threadattr);
	rc = pthread_attr_setdetachstate (&ec->threadattr,
				PTHREAD_CREATE_DETACHED);
	ec->heartbeatobject = strdup (EVENTTHREAD_HEARTBEAT_OBJECT);
	ec->heartbeatinterval = EVENTTHREAD_HEARTBEAT_INTERVAL;
	rc = pthread_create (&ec->thread,
			&ec->threadattr,
			eventclient_thread,
			ec);
	if (rc != 0) {
		comm_plugin_unload (&ec->commmodule);
		attribute_freeall (&ec->attributelist);
		squeue_free (&ec->eventqueue);
		free (ec);
		return rc;
	}
	*ecp = ec;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int     eventclient_uninit (EventClient **ecp)
{
	int		rc;
	struct timeval	tv;
	struct timespec	ts;
	EventClient	*ec = NULL;
	if (ecp == NULL || *ecp == NULL)
		return NETINFO_RC_INVALIDARG;
	ec = *ecp;
	/* Notify thread to close and exit */
	ec->threadflags |= EVENTTHREAD_FLAG_EXITTHREAD;
	rc = pthread_cond_signal (&ec->threadcond);

	/* Wait for the thread to signal it's finished */
	pthread_mutex_lock (&ec->queuemutex);
	gettimeofday (&tv, NULL);
	ts.tv_sec = tv.tv_sec + EVENTCLIENT_EXIT_WAIT_INTERVAL;
	ts.tv_nsec = tv.tv_usec * 1000;
	rc = pthread_cond_timedwait (&ec->threadcond,
					&ec->queuemutex,
					&ts);
	pthread_mutex_unlock (&ec->queuemutex);

	/* Unload plugin */
	rc = comm_plugin_unload (&ec->commmodule);
	/* Free structures */
	attribute_freeall (&ec->attributelist);
	squeue_free (&ec->eventqueue);
	free (ec);
	*ecp = NULL;	
	return 0;
}


NETINFO_API int     eventclient_emit_event (EventClient *ec, EventData *ed)
{
	int		rc;
	if (ec == NULL || ed == NULL)
		return NETINFO_RC_INVALIDARG;
	/* Add event to the squeue */
	pthread_mutex_lock (&ec->queuemutex);
	rc = squeue_item_enqueue (ec->eventqueue, ed);
	pthread_mutex_unlock (&ec->queuemutex);
	/* Notify the queue */
	rc = pthread_cond_signal (&ec->threadcond);
	return 0;
}


void	*eventclient_thread (void *arg)
{
	int		rc;
	int		count;
	struct timeval	tv;
	struct timespec	ts;
	EventClient	*ec = NULL;
	EventData	*ed = NULL;
	if (arg == NULL) 
		pthread_exit(NULL);
	ec = arg;
	ec->threadflags = 0;
	rc = eventdata_new (&ed);
	if (rc != NETINFO_RC_SUCCESS)
		pthread_exit(NULL);
	while (1) {
		/* If we've been signaled to exit, break the loop */
		if (ec->threadflags & EVENTTHREAD_FLAG_EXITTHREAD)
			break;

		/* Check connection and establish it, if required */
		rc = eventclient_test_and_connect (ec);

		/* Wait for an event or a timeout */
		pthread_mutex_lock (&ec->queuemutex);
		gettimeofday (&tv, NULL);
		ts.tv_sec = tv.tv_sec + ec->heartbeatinterval;
		ts.tv_nsec = tv.tv_usec * 1000;
		rc = pthread_cond_timedwait (&ec->threadcond,
						&ec->queuemutex,
						&ts);
		pthread_mutex_unlock (&ec->queuemutex);

		/* If we've been signaled to exit, break the loop */
		if (ec->threadflags & EVENTTHREAD_FLAG_EXITTHREAD)
			break;

		/* Emit the heartbeat, if it's time */
		if (rc == ETIMEDOUT) {
			/* Heartbeat */
			rc = eventdata_set (ed,
					EVENT_FLAG_HEARTBEAT,
					0,
					ec->heartbeatobject,
					"Alive",
					5);
			rc = eventclient_prepare_and_write_event(ec, ed);
		}

#ifdef WIN32
		/* FIXME: on w32 we have to handle EPIPE(WSAECONNABORT) */
		if (rc == EPIPE) {
			ec->threadflags -=
				(ec->threadflags & EVENTTHREAD_FLAG_CONNECTED);
			eventclient_test_and_connect (ec);
		}
#endif

		/* Process any queued items */
		rc = squeue_item_count(ec->eventqueue, &count);
		if (count != 0) {
			/* Handle event(s) */
			while (1) {
				pthread_mutex_lock (&ec->queuemutex);
				rc = squeue_item_dequeue (ec->eventqueue,
							ed);
				pthread_mutex_unlock (&ec->queuemutex);
				if (rc != 0)
					break;
				rc = eventclient_prepare_and_write_event(ec,
								ed);
				if (rc != 0) {
					/* TODO: Requeue the item */
					break;
				}
			}
		}
	}

	/* Process any remaining items in the queue, before exiting */
	rc = eventclient_test_and_connect (ec);
	rc = squeue_item_count(ec->eventqueue, &count);
	if (count != 0) {
		/* Handle event(s) */
		while (1) {
			pthread_mutex_lock (&ec->queuemutex);
			rc = squeue_item_dequeue (ec->eventqueue,
						ed);
			pthread_mutex_unlock (&ec->queuemutex);
			if (rc != 0)
				break;
			rc = eventclient_prepare_and_write_event(ec,
							ed);
		}
	}

	/* Clean up before exiting the thread */
	rc = eventdata_free (&ed);
	if (ec->commchannel != NULL) {
		rc = comm_close (&ec->commchannel, 0);
		ec->threadflags -=
			(ec->threadflags & EVENTTHREAD_FLAG_CONNECTED);
	}
	/* Notify the main thread that we're complete */
	rc = pthread_cond_signal (&ec->threadcond);
	return (NULL);
}


int     eventclient_prepare_and_write_event (EventClient *ec, EventData *ed)
{
	int		rc;
	int		bytes;
	if (ec == NULL || ed == NULL)
		return NETINFO_RC_INVALIDARG;
	rc = eventdata_to_networkorder (ed);
	rc = eventdata_sizeof (ed, &bytes);
	rc = comm_write (ec->commchannel,
			(char *)ed,
			&bytes);
	if (rc == 0) {
		if ((ec->clientflags & EC_FLAG_DEBUG) &&
		     ec->logfunction != NULL) {
			(*ec->logfunction)
				("(eventclient) DEBUG: comm_write: bytes=%lu rc=%d.\n",
				bytes,
				rc);
		}
		return rc;
	} else if (rc == EPIPE) {
		ec->threadflags -=
			(ec->threadflags & EVENTTHREAD_FLAG_CONNECTED);
		/* Try to re-establish the connection */
		rc = eventclient_test_and_connect (ec);
		if (rc == 0) {
			/* Try sending the event again */
			rc = comm_write (ec->commchannel,
					(char *)ed,
					&bytes);
			if (ec->logfunction != NULL && rc != 0) {
				(*ec->logfunction)
					("(eventclient) comm_write(2nd try): failed rc=%d.\n",
					rc);
			}
		} else {
			/* Log error, we couldn't re-establish */
			if (ec->logfunction != NULL) {
				(*ec->logfunction)
					("(eventclient) Failed to connect to the manager, this operation will be retried later.\n");
			}
		}
	} else {
		/* Log error, some other error occurred */
		ec->threadflags -=
			(ec->threadflags & EVENTTHREAD_FLAG_CONNECTED);
		if (ec->logfunction != NULL) {
			(*ec->logfunction)
				("(eventclient) comm_write: failed rc=%d.\n",
				rc);
		}
	}
	return rc;
}


NETINFO_API int     eventclient_configure_heartbeat (EventClient *ec,
                                        char *object,
                                        int interval)
{
	int		rc;
	if (ec == NULL)
		return NETINFO_RC_INVALIDARG;
	if (object != NULL) {
		if (ec->heartbeatobject != NULL)
			free (ec->heartbeatobject);
		ec->heartbeatobject = strdup (object);
	}
	if (interval != 0)
		ec->heartbeatinterval = interval;
	rc = pthread_cond_signal (&ec->threadcond);
	return NETINFO_RC_SUCCESS;
}


int	eventclient_test_and_connect (EventClient *ec)
{
	int		rc;
	if (ec == NULL)
		return NETINFO_RC_INVALIDARG;
	if (ec->threadflags & EVENTTHREAD_FLAG_CONNECTED)
		return NETINFO_RC_SUCCESS;
	if (ec->commchannel != NULL) {
		rc = comm_close (&ec->commchannel, 0);
	}
	rc = comm_connect (ec->commmodule,
			&ec->commchannel,
			ec->eventhost,
			ec->eventport);
	if (rc == 0) {
		ec->threadflags |= EVENTTHREAD_FLAG_CONNECTED;
		if (ec->logfunction != NULL) {
			(*ec->logfunction)
				("(eventclient) Connection to manager established.\n");
		}
	} else {
		/* We'll continue and try to reconnect */
		/* again on the next loop */
		if (ec->logfunction != NULL) {
			(*ec->logfunction)
				("(eventclient) Failed to connect to event sink. rc=%d.\n",
				rc);
		}
	}
	return rc;
}


NETINFO_API int eventclient_set_option (EventClient *ec,
                                        u_int32_t option,
                                        void *valuep)
{
	int	rc = NETINFO_RC_SUCCESS;
	if (ec == NULL || valuep == NULL)
		return EINVAL;
	switch (option) {
		case EC_OPTION_HEARBEAT_INTERVAL:
			if (*(uint32_t *)valuep != 0)
				ec->heartbeatinterval = *(uint32_t *)valuep;
			rc = pthread_cond_signal (&ec->threadcond);
			break;
		case EC_OPTION_HEARBEAT_OBJECTNAME:
			if ((char *)valuep != NULL) {
				if (ec->heartbeatobject != NULL)
					free (ec->heartbeatobject);
				ec->heartbeatobject = strdup ((char *)valuep);
			}
			rc = pthread_cond_signal (&ec->threadcond);
			break;
		case EC_OPTION_LOGGING_FUNCTION:
			ec->logfunction = (EventClientLogFunction)valuep;
			break;
		case EC_OPTION_ENABLE_DEBUG:
			ec->clientflags |= EC_FLAG_DEBUG;
			break;
		default:
			rc = EINVAL;
	}
	return rc;
}



