/************************************************************************/
/* netbuffer.c - Network Buffer Source					*/
/************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <netinfo/netinfo_portable.h>
#include <netinfo/netbuffer.h>

#define	RC_SUCCESS	0

NETINFO_API int netbuffer_new (NetBuffer **nbp)
{
	NetBuffer	*nb = NULL;
	if (nbp == NULL)
		return EINVAL;
	nb = calloc (1, sizeof (NetBuffer));
	if (nb == NULL)
		return ENOMEM;
	*nbp = nb;
	return RC_SUCCESS;
}


NETINFO_API int netbuffer_free (NetBuffer **nbp)
{
	NetBuffer	*nb = NULL;
	if (nbp == NULL || *nbp == NULL)
		return EINVAL;
	nb = *nbp;
	free (nb);
	*nbp = NULL;
	return RC_SUCCESS;
}


NETINFO_API void netbuffer_clear (NetBuffer *nb)
{
	if (nb == NULL)
		return;
/*	memset (nb, 0, sizeof (NetBuffer)); */
	nb->woffset = 0;
	nb->roffset = 0;
	return;
}


NETINFO_API int netbuffer_write (NetBuffer *nb, void *data, u_int32_t datalen)
{
	unsigned char *p = NULL;
	if (nb == NULL || data == NULL)
		return EINVAL;
	if (datalen == 0)
		return RC_SUCCESS;
	if (datalen > (NET_BUFFER_SIZE - nb->woffset))
		return ENOSPC;
	p = nb->buffer + nb->woffset;
	memcpy (p, data, datalen);
	nb->woffset += datalen;
	return RC_SUCCESS;
}


NETINFO_API int netbuffer_read (NetBuffer *nb, void *data, u_int32_t datalen)
{
	unsigned char *p = NULL;
	if (nb == NULL || data == NULL)
		return EINVAL;
	if (datalen == 0)
		return RC_SUCCESS;
	if (nb->woffset - nb->roffset == 0)
		return ENODATA;
	if (datalen > (nb->woffset - nb->roffset))
		datalen = nb->woffset - nb->roffset;
	p = nb->buffer + nb->roffset;
	memcpy (data, p, datalen);
	nb->roffset += datalen;
	return RC_SUCCESS;
}


NETINFO_API int netbuffer_bytes_in_buffer (NetBuffer *nb, u_int32_t *bytes)
{
	if (nb == NULL || bytes == NULL)
		return EINVAL;
	*bytes = nb->woffset - nb->roffset;
	return RC_SUCCESS;
}


NETINFO_API void netbuffer_dump (NetBuffer *nb)
{
	u_int32_t	i;
	u_int32_t	c;
	if (nb == NULL)
		return;
	fprintf (stdout,
		"\nNetBuffer Dump: nb=%p woffset=%d roffset=%d\n",
		nb,
		nb->woffset,
		nb->roffset);
	fprintf (stdout, "------------------------------");
	fprintf (stdout, "------------------------------\n");
	c = 0;
	for (i=0 ; i<nb->woffset ; i++) {
		if (nb->buffer[i] > 32 && nb->buffer[i] < 127)
			fprintf (stdout, " %c ", nb->buffer[i]);
		else 
			fprintf (stdout, "%02x ", nb->buffer[i]);
		c++;
		if (c == 16) {
			fprintf (stdout, "\n");
			c = 0;
		}
	}
	fprintf (stdout, "\n");
	return;
}


