

#include <string.h>
#include <errno.h>
#ifndef WIN32
#include <netinet/in.h>
#else
#include <winsock2.h>
#endif

#include <stdarg.h>

#include <netinfo/netinfo.h>
#include <netinfo/nmodule_p.h>
#include <netinfo/event_p.h>

/* v2 stuff */

NETINFO_API int netinfo_header_clear (NetinfoHeader *nh)
{
	if (nh == NULL)
		return EINVAL;
	memset (nh, 0, sizeof (NetinfoHeader));
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int netinfo_header_set_flags (NetinfoHeader *nh, u_int32_t flags)
{
	if (nh == NULL)
		return EINVAL;
	nh->flags = flags;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int netinfo_header_get_flags (NetinfoHeader *nh, u_int32_t *pflags)
{
	if (nh == NULL || pflags == NULL)
		return EINVAL;
	*pflags = nh->flags;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int netinfo_header_set_status (NetinfoHeader *nh, u_int32_t status)
{
	if (nh == NULL)
		return EINVAL;
	nh->status = status;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int netinfo_header_get_status (NetinfoHeader *nh, u_int32_t *pstatus)
{
	if (nh == NULL || pstatus == NULL)
		return EINVAL;
	*pstatus = nh->status;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int netinfo_header_set_datalen (NetinfoHeader *nh, u_int32_t datalen)
{
	if (nh == NULL)
		return EINVAL;
	nh->datalen = datalen;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int netinfo_header_get_datalen (NetinfoHeader *nh, u_int32_t *pdatalen)
{
	if (nh == NULL || pdatalen == NULL)
		return EINVAL;
	*pdatalen = nh->datalen;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int netinfo_header_set_rc (NetinfoHeader *nh, u_int32_t rc)
{
	if (nh == NULL)
		return EINVAL;
	nh->rc = rc;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int netinfo_header_get_rc (NetinfoHeader *nh, u_int32_t *prc)
{
	if (nh == NULL || prc == NULL)
		return EINVAL;
	*prc = nh->rc;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int netinfo_header_set_command (NetinfoHeader *nh, char *command)
{
	if (nh == NULL || command == NULL)
		return EINVAL;
	memset (nh->command, 0, NETINFO_V2_COMMAND_SIZE);
	strncpy (nh->command, command, NETINFO_V2_COMMAND_SIZE);
	nh->command[NETINFO_V2_COMMAND_SIZE-1] = 0;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int netinfo_header_get_command (NetinfoHeader *nh, char *command, int len)
{
	if (nh == NULL || command == NULL || len < NETINFO_V2_COMMAND_SIZE)
		return NETINFO_RC_INVALIDARG;
	strncpy (command, nh->command, NETINFO_V2_COMMAND_SIZE);
	command[NETINFO_V2_COMMAND_SIZE-1] = 0;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int netinfo_header_set_argdata (NetinfoHeader *nh, char *argdata, int len)
{
	if (nh == NULL || argdata == NULL)
		return NETINFO_RC_INVALIDARG;
	if (len > NETINFO_HEADER_ARGDATA_SIZE)
		return NETINFO_RC_INVALIDARG;
	memcpy (nh->argdata, argdata, len);
	nh->arglen = len;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int netinfo_header_get_argdata (NetinfoHeader *nh, char *argdata, int *plen)
{
	if (nh == NULL || argdata == NULL || plen == NULL)
		return NETINFO_RC_INVALIDARG;
	if (*plen < nh->arglen) {
		*plen = 0;
		return NETINFO_RC_INVALIDARG;
	}
	memcpy (argdata, nh->argdata, nh->arglen);
	*plen = nh->arglen;
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int netinfo_header_to_networkorder (NetinfoHeader *nh)
{
	if (nh == NULL)
		return EINVAL;
	nh->status = htonl (nh->status);
	nh->flags = htonl (nh->flags);
	nh->datalen = htonl (nh->datalen);
	nh->arglen = htonl (nh->arglen);
	nh->rc = htonl (nh->rc);
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int netinfo_header_to_hostorder (NetinfoHeader *nh)
{
	if (nh == NULL)
		return EINVAL;
	nh->status = ntohl (nh->status);
	nh->flags = ntohl (nh->flags);
	nh->datalen = ntohl (nh->datalen);
	nh->arglen = ntohl (nh->arglen);
	nh->rc = ntohl (nh->rc);
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int	vector_log_entry (ModuleInfo *mi, char *format, ...)
{
	va_list args;
	int rc;
	if (mi == NULL || format == NULL)
		return NETINFO_RC_INVALIDARG;
	if (mi->vector == NULL || mi->vector->log_entry == NULL)
		return NETINFO_RC_NOTIMPLEMENTED;
	va_start (args, format);
	rc = (*mi->vector->log_entry)(mi, format, args);
	va_end (args);
	return NETINFO_RC_SUCCESS;
}


NETINFO_API int	vector_emit_event (ModuleInfo *mi, EventData *ed)
{
	int rc;
	if (mi == NULL || ed == NULL)
		return NETINFO_RC_INVALIDARG;
	if (mi->vector == NULL || mi->vector->emit_event == NULL)
		return NETINFO_RC_NOTIMPLEMENTED;
	rc = (*mi->vector->emit_event)(mi, ed);
	return rc;
}


NETINFO_API int	vector_eventchain_open (ModuleInfo *mi, EventChain **ec)
{
	int rc;
	if (mi == NULL || ec == NULL)
		return NETINFO_RC_INVALIDARG;
	if (mi->vector == NULL || mi->vector->eventchain_open == NULL)
		return NETINFO_RC_NOTIMPLEMENTED;
	rc = (*mi->vector->eventchain_open)(mi, ec);
	return rc;
}


NETINFO_API int	vector_eventchain_close (EventChain **ec)
{
	int rc;
	EventChain *c = NULL;
	ModuleInfo *mi = NULL;
	if (ec == NULL ||*ec == NULL)
		return NETINFO_RC_INVALIDARG;
	c = *ec;
	mi = c->moduleinfo;
	if (mi == NULL)
		return NETINFO_RC_INVALIDARG;
	if (mi->vector == NULL || mi->vector->eventchain_close == NULL)
		return NETINFO_RC_NOTIMPLEMENTED;
	rc = (*mi->vector->eventchain_close)(ec);
	return rc;
}


NETINFO_API int	vector_eventchain_emit_event (EventChain *ec, EventData *ed)
{
	int rc;
	ModuleInfo *mi = NULL;
	if (ec == NULL || ec->moduleinfo == NULL || ed == NULL)
		return NETINFO_RC_INVALIDARG;
	mi = ec->moduleinfo;
	if (mi->vector == NULL || mi->vector->eventchain_emit_event == NULL)
		return NETINFO_RC_NOTIMPLEMENTED;
	rc = (*mi->vector->eventchain_emit_event)(ec, ed);
	return rc;
}


/* v2 stuff */


