/************************************************************************/
/* average.c - averaging structure and API                              */
/************************************************************************/
/*
 *  (c)2005 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
/************************************************************************/


#include <stdint.h>
#include <stdlib.h>
#include <errno.h>

#include <average.h>


Average *average_new (int avgtype, int avgcount)
{
        Average *a = NULL;
        a = calloc (1, sizeof (Average));
        if (a == NULL)
                return NULL;
        switch(avgtype) {
                case AVERAGE_TYPE_INTEGER32:
                case AVERAGE_TYPE_INTEGER64:
                case AVERAGE_TYPE_FLOAT:
                case AVERAGE_TYPE_DOUBLE:
                        break;
                default:
                        free (a);
                        return NULL;
        }
        a->type = avgtype;
        a->size = avgcount;
        a->average = calloc (avgcount, sizeof (AverageValue));
        if (a->average == NULL) {
                free (a);
                return NULL;
        }
        return a;
}


int average_free (Average *a)
{
        if (a == NULL)
                return EINVAL;
        if (a->average != NULL)
                free (a->average);
        free (a);
        return 0;
}


int average_push_value (Average *a, void *valuep)
{
        if (a == NULL || valuep == NULL)
                return EINVAL;
        switch(a->type) {
                case AVERAGE_TYPE_INTEGER32:
                        a->average[a->index].i32 = *(uint32_t *)valuep;
                case AVERAGE_TYPE_INTEGER64:
                        a->average[a->index].i64 = *(uint64_t *)valuep;
                case AVERAGE_TYPE_FLOAT:
                        a->average[a->index].f = *(float *)valuep;
                case AVERAGE_TYPE_DOUBLE:
                        a->average[a->index].d = *(double *)valuep;
                        break;
                default:
                        return EINVAL;
        }
        a->index++;
        if (a->index == a->size)
                a->index = 0;
        return 0;
}


int average_get_average (Average *a, void *valuep)
{
        int32_t         i32;
        int64_t         i64;
        float           f;
        double          d;
        int             i;
        if (a == NULL || valuep == NULL)
                return EINVAL;
        switch(a->type) {
                case AVERAGE_TYPE_INTEGER32:
                        i32 = 0;
                        for (i=0 ; i<a->size ; i++)
                                i32 += a->average[i].i32;
                        *(int32_t *)valuep = i32 / a->size;
                        break;
                case AVERAGE_TYPE_INTEGER64:
                        i64 = 0;
                        for (i=0 ; i<a->size ; i++)
                                i64 += a->average[i].i64;
                        *(int64_t *)valuep = i64 / a->size;
                        break;
                case AVERAGE_TYPE_FLOAT:
                        f = 0;
                        for (i=0 ; i<a->size ; i++)
                                f += a->average[i].f;
                        *(float *)valuep = f / (float)a->size;
                        break;
                case AVERAGE_TYPE_DOUBLE:
                        d = 0;
                        for (i=0 ; i<a->size ; i++)
                                d += a->average[i].d;
                        *(double *)valuep = d / (double)a->size;
                        break;
        }
        return 0;
}


