/************************************************************************/
/* fsobject.h - Filesystem Object Header 				*/
/************************************************************************/
/*
 * (c) 2002-2003 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/

#ifndef _FSOBJECT_H_
#define _FSOBJECT_H_

#ifndef uint32
#define uint32	unsigned int
#endif

#ifndef uint64
#define uint64	unsigned long long
#endif

#define MAX_PATH_LEN	1024

typedef struct _FSObjectAcl	FSObjectAcl;
typedef struct _FSObjectAttr	FSObjectAttr;
typedef struct _FSObject	FSObject;

#define ACL_PERM_READ		0x00000004
#define ACL_PERM_WRITE		0x00000002
#define ACL_PERM_EXECUTE	0x00000001

#define ACL_TAG_USER_OBJ	0x00000001
#define ACL_TAG_USER		0x00000002
#define ACL_TAG_GROUP_OBJ	0x00000004
#define ACL_TAG_GROUP		0x00000008
#define ACL_TAG_MASK		0x00000010
#define ACL_TAG_OTHER		0x00000020

#ifndef ACL_TYPE_ACCESS
#define ACL_TYPE_ACCESS		0x00008000
#endif
#ifndef ACL_TYPE_DEFAULT
#define ACL_TYPE_DEFAULT	0x00004000
#endif

struct _FSObjectAcl {
	uint32 acl_tag;		/* The ACL type */
	uint32 acl_perm;	/* The ACL permissions */
	uint32 acl_id;		/* The ACL id */
	char *acl_idname;	/* The user/group/whatever name */
	};

struct _FSObjectAttr {
	char *attr_name;
	char *attr_value;
	};


#define OBJECT_TYPE_FILE	0x00000001
#define OBJECT_TYPE_SYMLINK	0x00000002
#define OBJECT_TYPE_HARDLINK	0x00000004
#define OBJECT_TYPE_DIRECTORY	0x00000008
#define OBJECT_TYPE_CHARDEV	0x00000010
#define OBJECT_TYPE_BLOCKDEV	0x00000020
#define OBJECT_TYPE_FIFO	0x00000040
#define OBJECT_TYPE_SOCKET	0x00000080

struct _FSObject {
	char *object_path; /* path to object */
	uint32 object_type;
	uint32 object_mode;
	uint32 object_links;
	uint64 object_size;
	uint64 object_blocksize;
	uint64 object_blocks;
	uint64 object_atime;
	uint64 object_mtime;
	uint64 object_ctime;
	FSObjectAcl *object_acllist;
	};	


#define FSOBJECT_GET_OWNER_MODE(x)	(((x)->object_mode & 0x01c0) >> 6 )
#define FSOBJECT_GET_GROUP_MODE(x)	((x->object_mode & 0x0038) >> 3 )
#define FSOBJECT_GET_OTHER_MODE(x)	(x->object_mode & 0x0007)


int fsobject_stat(char *objectpath, FSObject **pfso);
int fsobject_get_acllist(char *objectpath, FSObjectAcl ** pfsoa);
int fsobject_get_attrlist(char *objectpath, FSObjectAttr ** pfsoa);
int fs_object_free(FSObject **object);

#endif



