/************************************************************************/
/* fstraverse.h - Filesystem Traversal Library Header 			*/
/************************************************************************/
/*
 * (c) 2002-2003 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/


#ifndef _FSTRAVERSE_H_
#define _FSTRAVERSE_H_

/************************************************************************/
/* Includes								*/
/************************************************************************/
#include "fsobject.h"
#include "stack.h"


/************************************************************************/
/* Definitions								*/
/************************************************************************/
#ifndef uint32
#define uint32	unsigned int
#endif

#ifndef uint64
#define uint64	unsigned long long
#endif

#define MAX_PATH_LEN	1024


/************************************************************************/
/* Portability Definitions						*/
/************************************************************************/
#ifndef WIN32
#define DIRDELIMITER	"/"
#else
#define DIRDELIMITER	"\\"
#endif


/************************************************************************/
/* Structure Declarations						*/
/************************************************************************/
typedef struct _FSTrav		FSTrav;
typedef int (*FSTravCallback)(FSTrav *fst, FSObject *fso);

/* Recurse through the tree? */
#define FST_FLAG_RECURSIVE		0x00000001

/* Show files starting with a '.'? */
#define FST_FLAG_SHOWHIDDEN		0x00000002

/* Was the last entry a directory? Used to known when to drop into a dir */
#define FST_FLAG_LASTENTRYDIR		0x10000000


struct _FSTrav {
	char *workpath;			/* The path we started from */
	char *currpath;			/* Current directory path */
	void *impl_data;		/* implementation data (current)*/
	sstack_t *stack;		/* implementation data stack */
	FSTravCallback callback;	/* Callback */
	uint32 flags;			/* flags */
	};


/************************************************************************/
/* Function Prototypes							*/
/************************************************************************/
FSTrav * fs_trav_open(char *path, uint32 flags);
int fs_trav_next_entry(FSTrav *fst, FSObject **fso);
int fs_trav_close(FSTrav *fst);

int fs_trav_set_callback(FSTrav *fst, FSTravCallback *cb);
int fs_trav_traverse(FSTrav *fst);

int fs_path_stat(char *objectpath, FSObject **pfso);
int fs_path_get_acllist(char *objectpath, FSObjectAcl ** pfsoa);
int fs_path_get_attrlist(char *objectpath, FSObjectAttr ** pfsoa);


#endif



