/************************************************************************/
/* nieh_log.c - Netinfoevtd Event Logger plugin				*/
/************************************************************************/
/*
 *  (c)2005 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
/************************************************************************/



#ifdef WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <time.h>


#include <netinfo/netinfo.h>
#include <netinfo/event.h>

#define MODULE_NAME             "nieh_log"
#define MODULE_VERSION          "0.1.0"


#define NAME_SIZE		64
#define BUFFER_SIZE		512

#define	CONFIG_DELIMITERS	",\r\n"
#define LOGFILE_DEFAULT		"netinfo-events.log"

#ifndef WIN32
#define	DIR_DELIMITER		"/"
#else
#define	DIR_DELIMITER		"\\"
#endif

#define	FLAG_FLUSH		1
#define FLAG_REPLACE_LFCHARS	2



/* Global Variables */
unsigned int	log_flags;
char		*log_file;
FILE		*log_fd;


int module_init (char *args)
{
	char	*argtokp;
	char	*str = NULL;
	log_flags = 0;
	log_file = NULL;
	log_fd = NULL;
	if (args == NULL)
		return NETINFO_RC_INVALIDARG;
	str = strtok_r (args, CONFIG_DELIMITERS, &argtokp);
	while (1) {
		if (str == NULL)
			break;
		if (!strcmp (str, "FlushLog")) {
			log_flags |= FLAG_FLUSH;
		} else if (strstr (str, "replacectrlchars=")) {
			str = strtok_r (str, "=", &argtokp);
			if (atoi((strtok_r (NULL,
			    CONFIG_DELIMITERS, &argtokp))) == 1)
				log_flags |= FLAG_REPLACE_LFCHARS;
		} else { 
			log_file = strdup (str);
		}
		str = strtok (NULL, CONFIG_DELIMITERS);
	}
	if (log_file == NULL) {
		log_file = strdup (LOGDIR DIR_DELIMITER LOGFILE_DEFAULT);
	}
	log_fd = fopen (log_file, "a");
	if (log_fd == NULL) {
		free (log_file);
		return NETINFO_RC_INVALIDARG;
	}
	return NETINFO_RC_SUCCESS;
}


int module_cleanup (void)
{
	if (log_fd != NULL)
		fclose (log_fd);
	if (log_file != NULL)
		free (log_file);
	return NETINFO_RC_SUCCESS;
}


int handle_event (EventData *event)
{
	int		i;
	int		rc;
	int		len;
	u_int32_t	eventrc;
	u_int32_t	eventflags;
	u_int64_t	eventtime;
	char		hostname[NAME_SIZE];
	char		objectname[NAME_SIZE];
	char		data[BUFFER_SIZE];

	if (event == NULL || log_fd == NULL)
		return NETINFO_RC_INVALIDARG;
	memset (hostname, 0, NAME_SIZE);
	memset (objectname, 0, NAME_SIZE);
	memset (data, 0, BUFFER_SIZE);
	len = NAME_SIZE;
	rc = eventdata_get_hostname (event, hostname, &len);
	len = NAME_SIZE;
	rc = eventdata_get_object (event, objectname, &len);
	rc = eventdata_get_rc (event, &eventrc);
	rc = eventdata_get_flags (event, &eventflags);
	rc = eventdata_get_timestamp (event, &eventtime);
	len = BUFFER_SIZE;
	rc = eventdata_get_data (event, data, &len);

	// If enabled, replace line control characters with a space
	if (log_flags & FLAG_REPLACE_LFCHARS) {
		for (i=0 ; i<len ; i++) {
			if (data[i] == '\n' || data[i] == '\r') {
				data[i] = ' ';
			}
		}
	}

	rc = fprintf (log_fd,
			"%lu hostname=%s;object=%s;eventrc=%lu;eventflags=%lu;eventtime=%llu;data=%s\n",
			time(NULL),
			hostname,
			objectname,
			(unsigned long)eventrc,
			(unsigned long)eventflags,
			(unsigned long long)eventtime,
			data);
	if (log_flags & FLAG_FLUSH)
		fflush (log_fd);
	return NETINFO_RC_SUCCESS;
}


