/************************************************************************/
/* nieh_nagios.c - Netinfoevtd Nagios Event plugin			*/
/************************************************************************/
/*
 *  (c)2005 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
/************************************************************************/



#ifdef WIN32
#include <windows.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/select.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <time.h>


#include <netinfo/netinfo.h>
#include <netinfo/event.h>


#define	MODULE_NAME		"nieh_nagios"
#define	MODULE_VERSION		"0.1.1"


#define BUFFER_SIZE		512
#define NAME_SIZE		80
#define CMD_FILE_DEFAULT	"/var/spool/nagios/nagios.cmd"

#define ADDED_SELECT_20050827
#ifdef ADDED_SELECT_20050827
#define WAIT_SECONDS		10
#endif

char	*nagios_cmd_file;
/* FILE	*nagios_cmd_fd;	*/
int	nagios_cmd_fd;


int module_init (char *args)
{
	nagios_cmd_fd = -1;
	if (args == NULL)
		nagios_cmd_file = strdup (CMD_FILE_DEFAULT);
	else
		nagios_cmd_file = strdup (args);
	nagios_cmd_fd = open (nagios_cmd_file, O_WRONLY | O_NONBLOCK);
	if (nagios_cmd_fd == -1) {
		free (nagios_cmd_file);
		return NETINFO_RC_INVALIDARG;
	}
	return NETINFO_RC_SUCCESS;
}


int module_cleanup (void)
{
	if (nagios_cmd_fd != -1)
		close (nagios_cmd_fd);
	if (nagios_cmd_file != NULL)
		free (nagios_cmd_file);
	return NETINFO_RC_SUCCESS;
}


int handle_event (EventData *event)
{
	int		rc;
	int		len;
	int		bytes;
	u_int32_t	eventrc;
	u_int32_t	eventflags;
#ifdef ADDED_SELECT_20050827
	fd_set		wfds;
	struct timeval	waittv;
#endif
	char		hostname[NAME_SIZE];
	char		objectname[NAME_SIZE];
	char		data[BUFFER_SIZE];
	char		output[BUFFER_SIZE];
	char		*p = NULL;

	if (event == NULL || nagios_cmd_fd == -1)
		return NETINFO_RC_INVALIDARG;

	/* Only interested in the (old-style) performance events */
	rc = eventdata_get_flags (event, &eventflags);
	if (eventflags != 0)
		return NETINFO_RC_SUCCESS;

/*
	memset (hostname, 0, BUFFER_SIZE);
	memset (objectname, 0, BUFFER_SIZE);
	memset (data, 0, BUFFER_SIZE);
*/
	hostname[0] = 0;
	objectname[0] = 0;
	len = NAME_SIZE;
	rc = eventdata_get_hostname (event, hostname, &len);
	len = NAME_SIZE;
	rc = eventdata_get_object (event, objectname, &len);
	len = BUFFER_SIZE;
	rc = eventdata_get_data (event, data, &len);
	data[len] = 0;
	data[BUFFER_SIZE-1] = 0;
	rc = eventdata_get_rc (event, &eventrc);
	p = strtok (hostname, ".");
	len = snprintf (output,
			BUFFER_SIZE,
			"[%lu] PROCESS_SERVICE_CHECK_RESULT;%s;%s;%lu;%s\n",
			time(NULL),
			p,
			objectname,
			(unsigned long)eventrc,
			data);
	bytes = len;
	p = output;
	while (bytes > 0) {
		rc = write (nagios_cmd_fd, p, bytes);
		if (rc == -1) {
#ifdef ADDED_SELECT_20050827
			if (errno == EAGAIN) {
				FD_ZERO (&wfds);
				FD_SET (nagios_cmd_fd, &wfds);
				waittv.tv_sec = WAIT_SECONDS;
				waittv.tv_usec = 0;
				rc = select (nagios_cmd_fd+1,
						NULL,
						&wfds,
						NULL,
						&waittv);
				continue;
			} else if (errno == EBADF) {
				close (nagios_cmd_fd);
				nagios_cmd_fd = open (nagios_cmd_file,
						O_WRONLY | O_NONBLOCK);
				if (nagios_cmd_fd == -1) {
					return errno;
				} else {
					continue;
				}
			} else {
				return NETINFO_RC_MODULEERROR;
			}
#else
			return NETINFO_RC_MODULEERROR;
#endif
		}
		bytes -= rc;
		p += rc;
	}
	return NETINFO_RC_SUCCESS;
}


