/************************************************************************/
/* nieh_oneshot2mysql.c - Netinfoevtd Event Oneshot->MySQL plugin	*/
/************************************************************************/
/*
 *  (c)2006 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
/************************************************************************/



#ifdef WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <time.h>


#include <netinfo/netinfo.h>
#include <netinfo/event.h>

#include <mysql/mysql.h>

#define MODULE_NAME             "nieh_oneshot2mysql"
#define MODULE_VERSION          "0.1.0"

#define NAME_SIZE		64
#define BUFFER_SIZE		512
#define SQL_BUFFER_SIZE		2048

#define	CONFIG_DELIMITERS	",\r\n"
#define LOGFILE_DEFAULT		"netinfo-oneshot2mysql.log"

#define	NETINFO_EVENT_FLAG_ONESHOT	0x00000001

#ifndef WIN32
#define	DIR_DELIMITER		"/"
#else
#define	DIR_DELIMITER		"\\"
#endif


/* Global Variables */
unsigned int	log_flags;
char		*log_file;
FILE		*log_fd;
char		*dbhost;
char		*dbuser;
char		*dbpass;
char		*dbname;
int		dbport;
MYSQL		*myconn;

/* Function prototypes */
int module_mysql_connect (void);
int module_mysql_disconnect (void);


int module_init (char *args)
{
	int	rc;
	char	*str = NULL;
	char	*argtokp = NULL;
	char	*valtokp = NULL;
	log_flags = 0;
	log_file = NULL;
	log_fd = NULL;
	dbhost = NULL;
	dbuser = NULL;
	dbpass = NULL;
	dbname = NULL;
	dbport = 3306;
	myconn = NULL;
	if (args == NULL)
		return NETINFO_RC_INVALIDARG;
	str = strtok_r (args, CONFIG_DELIMITERS, &argtokp);
	while (1) {
		if (str == NULL)
			break;
		if (strstr (str, "dbhost=")) {
			str = strtok_r (str, "=", &valtokp);
			dbhost = strdup (strtok_r (NULL, CONFIG_DELIMITERS, &valtokp));
		} else if (strstr (str, "dbuser=")) {
			str = strtok_r (str, "=", &valtokp);
			dbuser = strdup (strtok_r (NULL, CONFIG_DELIMITERS, &valtokp));
		} else if (strstr (str, "dbpass=")) {
			str = strtok_r (str, "=", &valtokp);
			dbpass = strdup (strtok_r (NULL, CONFIG_DELIMITERS, &valtokp));
		} else if (strstr (str, "dbname=")) {
			str = strtok_r (str, "=", &valtokp);
			dbname = strdup (strtok_r (NULL, CONFIG_DELIMITERS, &valtokp));
		} else if (strstr (str, "dbport=")) {
			str = strtok_r (str, "=", &valtokp);
			str = strtok_r (NULL, CONFIG_DELIMITERS, &valtokp);
			dbport = atoi(str);
		} else { 
			log_file = strdup (str);
		}
		str = strtok_r (NULL, CONFIG_DELIMITERS, &argtokp);
	}
	if (dbhost == NULL || dbuser == NULL || dbpass == NULL || dbname == NULL)
		return NETINFO_RC_INVALIDARG;
	if (log_file == NULL) {
		log_file = strdup (LOGDIR DIR_DELIMITER LOGFILE_DEFAULT);
	}
	log_fd = fopen (log_file, "a");
	if (log_fd == NULL) {
		free (log_file);
		return NETINFO_RC_INVALIDARG;
	}
	rc = module_mysql_connect();
	if (rc != NETINFO_RC_SUCCESS)
		return rc;
	return NETINFO_RC_SUCCESS;
}


int module_cleanup (void)
{
	int	rc;
	rc = module_mysql_disconnect();
	if (log_fd != NULL)
		fclose (log_fd);
	if (log_file != NULL)
		free (log_file);
	if (dbhost != NULL)
		free (dbhost);
	if (dbname != NULL)
		free (dbname);
	if (dbuser != NULL)
		free (dbuser);
	if (dbpass != NULL)
		free (dbpass);
	return NETINFO_RC_SUCCESS;
}


int handle_event (EventData *event)
{
	int		rc;
	int		len;
	int		esclen;
	int		sqllen;
	u_int32_t	eventrc;
	u_int32_t	eventflags;
	u_int64_t	eventtimestamp;
	char		hostname[NAME_SIZE];
	char		objectname[NAME_SIZE];
	char		data[BUFFER_SIZE];
	char		escdata[BUFFER_SIZE * 2 + 1];
	char		sql[SQL_BUFFER_SIZE];
	char		*p = NULL;

	if (event == NULL)
		return NETINFO_RC_INVALIDARG;
	/*
	memset (hostname, 0, NAME_SIZE);
	memset (objectname, 0, NAME_SIZE);
	memset (data, 0, BUFFER_SIZE);
	*/
	rc = eventdata_get_flags (event, &eventflags);

	/* We only want to process Oneshot(trap) events */
	if ( !(eventflags & NETINFO_EVENT_FLAG_ONESHOT))
		return NETINFO_RC_SUCCESS;

	len = NAME_SIZE;
	rc = eventdata_get_hostname (event, hostname, &len);
	p = strstr(hostname, ".");
	if (p != NULL)
		*p = 0;
	len = NAME_SIZE;
	rc = eventdata_get_object (event, objectname, &len);
	len = BUFFER_SIZE;
	rc = eventdata_get_data (event, data, &len);
	rc = eventdata_get_rc (event, &eventrc);
/*	rc = eventdata_get_timestamp (event, &eventtimestamp); */

	esclen = mysql_real_escape_string (myconn,
					escdata,
					data,
					len);
	sqllen = snprintf (sql,
		SQL_BUFFER_SIZE,
		"insert into alerts (%s) values ('%s','%s',%d,%d,%d,'%s')",
		"hostname,objectname,timestamp,flags,rc,data",
		hostname,
		objectname,
		(int)time (NULL),
		eventflags,
		eventrc,
		escdata);
	rc = mysql_real_query (myconn, sql, sqllen);
	if (rc != 0) {
		fprintf (log_fd,
			"%lu mysql_real_query(%s): rc=%d (%s)\n",
			time(NULL),
			sql,
			rc,
			mysql_error (myconn));
		fprintf (log_fd,
			"%lu Attempting to reconnect...\n",
			time (NULL));
		rc = module_mysql_disconnect();
		rc = module_mysql_connect();
		fprintf (log_fd,
			"%lu Reconnect successful.\n",
			time (NULL));
		if (rc == NETINFO_RC_SUCCESS){
			rc = mysql_real_query (myconn, sql, sqllen);
			fprintf (log_fd,
				"%lu Resubmit (after reconnect) successful.\n",
				time (NULL));
		}
	}
	return rc;
}


int module_mysql_connect (void)
{
	int	rc;
	myconn = mysql_init (NULL);
	if (myconn == NULL)
		return NETINFO_RC_MEMORYERROR;
/*	mysql_option (myconn, MYSQL_OPT_RECONNECT, 1); */
	if (mysql_real_connect (myconn,
				dbhost,
				dbuser,
				dbpass,
				dbname,
				dbport,
				NULL,
				0) == NULL) {
		rc = mysql_errno(myconn);
		fprintf (log_fd,
			"%lu mysql_connect(): rc=%d (%s)\n",
			time(NULL),
			rc,
			mysql_error (myconn));
		mysql_close (myconn);
		return rc;
		
	}
	fprintf (log_fd,
		"%lu mysql_connect(): successfully connected.\n",
		time(NULL));
	return NETINFO_RC_SUCCESS;
}


int module_mysql_disconnect (void)
{
	mysql_close (myconn);
	fprintf (log_fd,
		"%lu mysql_disconnect(): disconnected.\n",
		time(NULL));
	return NETINFO_RC_SUCCESS;
}


