/************************************************************************/
/* nieh_oneshot2trap.c - Netinfoevtd Event Oneshot->trap plugin		*/
/************************************************************************/
/*
 *  (c)2007 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
/************************************************************************/



#ifdef WIN32
#include <windows.h>
#else
#include <unistd.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <time.h>


#include <netinfo/netinfo.h>
#include <netinfo/event.h>

#define MODULE_NAME			"nieh_oneshot2trap"
#define MODULE_VERSION			"0.1.1"

#define NAME_SIZE			64
#define BUFFER_SIZE			512

#define	CONFIG_DELIMITERS		",\r\n"
#define LOGFILE_DEFAULT			"netinfo-oneshot2trap.log"

#define	NETINFO_EVENT_FLAG_ONESHOT	0x00000001

#define	NETINFO_TRAP_OID_DEFAULT	"netinfo.event.trap"

#define MODULE_FLAG_DEBUG		0x00000001
#define MODULE_FLAG_REPLACE_LFCHARS	0x00000002

#ifndef WIN32
#define	DIR_DELIMITER			"/"
#else
#define	DIR_DELIMITER			"\\"
#endif


/* Global Variables */
unsigned int	module_flags;
char		*log_file;
FILE		*log_fd;
char		*trap_directory;
char		*trap_oid;
unsigned int	trap_count;

/* Function prototypes */
int cleanup (void);


int module_init (char *args)
{
	int		rc;
	struct stat	st;
	char		*str = NULL;
	char		*argtokp = NULL;
	char		*valtokp = NULL;
	module_flags = 0;
	log_file = NULL;
	log_fd = NULL;
	trap_directory = NULL;
	trap_oid = NULL;
	trap_count = 0;
	if (args == NULL)
		return NETINFO_RC_INVALIDARG;
	str = strtok_r (args, CONFIG_DELIMITERS, &argtokp);
	while (1) {
		if (str == NULL)
			break;
		if (strstr (str, "trapdir=")) {
			str = strtok_r (str, "=", &valtokp);
			trap_directory = strdup (strtok_r (NULL,
						CONFIG_DELIMITERS, &valtokp));
		} else if (strstr (str, "trapoid=")) {
			str = strtok_r (str, "=", &valtokp);
			trap_oid = strdup (strtok_r (NULL,
						CONFIG_DELIMITERS, &valtokp));
		} else if (strstr (str, "debug=")) {
			str = strtok_r (str, "=", &valtokp);
			if (atoi((strtok_r (NULL,
					CONFIG_DELIMITERS, &valtokp))) == 1)
				module_flags |= MODULE_FLAG_DEBUG;
		} else if (strstr (str, "replacectrlchars=")) {
			str = strtok_r (str, "=", &valtokp);
			if (atoi((strtok_r (NULL,
					CONFIG_DELIMITERS, &valtokp))) == 1)
				module_flags |= MODULE_FLAG_REPLACE_LFCHARS;
		} else { 
			log_file = strdup (str);
		}
		str = strtok_r (NULL, CONFIG_DELIMITERS, &argtokp);
	}
	if (log_file == NULL) {
		log_file = strdup (LOGDIR DIR_DELIMITER LOGFILE_DEFAULT);
	}
	log_fd = fopen (log_file, "a");
	if (log_fd == NULL) {
		cleanup ();
		return NETINFO_RC_INVALIDARG;
	}
	fprintf (log_fd, "%s Version %s\n", MODULE_NAME, MODULE_VERSION);
	if (trap_oid == NULL)
		trap_oid = strdup (NETINFO_TRAP_OID_DEFAULT);
	if (trap_directory == NULL) {
		fprintf (log_fd,
			"Error: trap_directory was not specified.\n");
		cleanup ();
		return NETINFO_RC_INVALIDARG;
	}
	rc = stat (trap_directory, &st);
	if (rc == -1) {
		rc = errno;
		fprintf (log_fd,
			"Error: trap_directory stat(): rc=%d(%s)\n",
			rc,
			strerror (rc));
		cleanup ();
		return rc;
	}
	if (S_ISDIR(st.st_mode) == 0) {
		fprintf (log_fd,
			"Error: trap_directory does not point to a directory.\n");
		cleanup ();
		return rc;
	}
	fprintf (log_fd, "Parameter: trap_directory=%s\n", trap_directory);
	fprintf (log_fd, "Config: module_flags=%d\n", module_flags);
	fprintf (log_fd, "Module initialized successfully.\n");
	return NETINFO_RC_SUCCESS;
}


int module_cleanup (void)
{
	int	rc;
	fprintf (log_fd, "Unloading module.\n");
	rc = cleanup ();
	return NETINFO_RC_SUCCESS;
}


int handle_event (EventData *event)
{
	int		i;
	int		rc;
	int		len;
	u_int32_t	eventrc;
	u_int32_t	eventflags;
	/* u_int64_t	eventtimestamp; */
	time_t		ct;
	struct tm	lt;
	char		hostname[NAME_SIZE];
	char		objectname[NAME_SIZE];
	char		data[BUFFER_SIZE];
	char		trappath[BUFFER_SIZE];
	FILE		*tfd = NULL;

	if (event == NULL)
		return NETINFO_RC_INVALIDARG;
	/*
	memset (hostname, 0, NAME_SIZE);
	memset (objectname, 0, NAME_SIZE);
	memset (data, 0, BUFFER_SIZE);
	*/
	rc = eventdata_get_flags (event, &eventflags);
	/* We only want to process Oneshot(trap) events */
	if ( !(eventflags & NETINFO_EVENT_FLAG_ONESHOT))
		return NETINFO_RC_SUCCESS;
	len = NAME_SIZE;
	rc = eventdata_get_hostname (event, hostname, &len);
	hostname[len] = 0;
/*	p = strstr(hostname, ".");
	if (p != NULL)
		*p = 0;
*/
	len = NAME_SIZE;
	rc = eventdata_get_object (event, objectname, &len);
	objectname[len] = 0;
	len = BUFFER_SIZE;
	rc = eventdata_get_data (event, data, &len);
	data[len] = 0;
	rc = eventdata_get_rc (event, &eventrc);
/*	rc = eventdata_get_timestamp (event, &eventtimestamp); */

	// If enabled, replace line control characters with a space
	if (module_flags & MODULE_FLAG_REPLACE_LFCHARS) {
		for (i=0 ; i<len ; i++) {
			if (data[i] == '\n' || data[i] == '\r') {
				data[i] = ' ';
			}
		}
	}

	rc = snprintf (trappath,
			BUFFER_SIZE,
			"%s/trap-%d-%d",
			trap_directory,
			(int)time (NULL),
			trap_count);
	tfd = fopen (trappath, "w");
	if (tfd != NULL) {
		if (module_flags & MODULE_FLAG_DEBUG) {
			fprintf (log_fd,
				"Processing event hostname='%s' object='%s' rc=%d flags=%d\n",
				hostname,
				objectname,
				eventrc,
				eventflags);
		}
		ct = time (NULL);
		localtime_r (&ct, &lt);
		rc = fprintf (tfd, "DATE: %04d-%02d-%02d\n",
				lt.tm_year+1900,
				lt.tm_mon+1,
				lt.tm_mday);
		rc = fprintf (tfd, "TIME: %02d:%02d:%02d\n",
				lt.tm_hour,
				lt.tm_min,
				lt.tm_sec);
		rc = fprintf (tfd,
				"TIMESTAMP: %d\n",
				(int)ct);
		rc = fprintf (tfd,
				"HOST: %s\n",
				hostname);
		rc = fprintf (tfd,
				"TRAP: SNMPv2-MIB::snmpTrapOID.0 %s\n",
				trap_oid);
		rc = fprintf (tfd,
				"TRAP: netinfo.event.hostname %s\n",
				hostname);
		rc = fprintf (tfd,
				"TRAP: netinfo.event.objectname %s\n",
				objectname);
		rc = fprintf (tfd,
				"TRAP: netinfo.event.flags %d\n",
				eventflags);
		rc = fprintf (tfd,
				"TRAP: netinfo.event.rc %d\n",
				eventrc);
/*		rc = fprintf (tfd, "TRAP: netinfo.event.timestamp %llu\n",
				eventtimestamp); */
		rc = fprintf (tfd,
				"TRAP: netinfo.event.data %s\n",
				data);
		fclose (tfd);
	} else {
		rc = errno;
		fprintf (log_fd,
			"Error: trap file fopen(): rc=%d(%s)\n",
			rc,
			strerror (rc));
	}
	trap_count++;
	return rc;
}


int cleanup (void)
{
	if (log_fd != NULL)
		fclose (log_fd);
	if (log_file != NULL)
		free (log_file);
	if (trap_directory != NULL)
		free (trap_directory);
	if (trap_oid != NULL)
		free (trap_oid);
	return NETINFO_RC_SUCCESS;
}



