/************************************************************************/
/* pcrelog.h - PCRE Log parsing library header				*/
/************************************************************************/
/*
 *  (c)2008 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
/************************************************************************/


#ifndef	_PCRELOG_H_
#define	_PCRELOG_H_



/* MatchTemplate types */
#define MT_TYPE_STRING          1
#define MT_TYPE_BACKREF         2

/* InputSource state definitions */
#define IS_STATE_OPEN           0x00000001

/* InputSource flag definitions */
#define IS_FLAG_NOREOPEN        0x00000001

/* Library specific error codes */
#define EROTATED                10010

/* Structure Prototypes */
typedef struct	_MatchData		MatchData;
typedef struct	_MatchRuleAttribute	MatchRuleAttribute;
typedef struct	_MatchRule		MatchRule;
typedef struct	_MatchGroup		MatchGroup;
typedef struct	_MatchTemplate		MatchTemplate;
typedef struct	_InputSource		InputSource;


/* Callback Function Prototypes */
typedef int (*MatchActionCallback)(MatchRule *rule, MatchData *matchdata);

/* Function Prototypes */
int matchrule_new (MatchRule **mrp, pcre *p, MatchActionCallback action,
                   void *actionarg);
int matchrule_free (MatchRule **mrp);
int matchrule_set_id (MatchRule *mr, unsigned int id);
int matchrule_get_id (MatchRule *mr, unsigned int *id);
int matchrule_set_desc (MatchRule *mr, char *desc);
int matchrule_get_desc (MatchRule *mr, char *desc, int desclen);
int matchrule_enable_match_interval (MatchRule *mr,
                                unsigned int intvseconds,
                                unsigned int intvmatches);
int matchrule_disable_match_interval (MatchRule *mr);
int matchrule_get_match_interval (MatchRule *mr,
                                unsigned int *intvsecondsp,
                                unsigned int *intvmatchesp);
int matchrule_set_message (MatchRule *mr, char *message);
int matchrule_set_argument (MatchRule *mr, void *arg);
int matchrule_get_argument (MatchRule *mr, void **argp);
int matchrule_set_attribute (MatchRule *mr, char *name, char *value);
MatchRuleAttribute *matchrule_get_attribute (MatchRule *mr, char *name);
char *matchrule_get_attribute_value (MatchRule *mr, char *name);
int matchrule_delete_attribute (MatchRule *mr, char *name);
int matchrule_construct_message (MatchRule *mr, MatchData *md, char *buffer,
                                int *buflenp);

int matchgroup_new (MatchGroup **mgp, char *name, char *prematch);
int matchgroup_free (MatchGroup **mgp);
int matchgroup_set_action_callback (MatchGroup *mg, MatchActionCallback maf,
					void *callbackarg);
int matchgroup_add_rule (MatchGroup *mg, MatchRule *mr);
int matchgroup_load_rule_file (MatchGroup *mg, char *rulefile, int filetype);

int inputsource_new (InputSource **isp, char *name, char *path, int buflen);
int inputsource_free (InputSource **isp);
int inputsource_set_matchgroup (InputSource *is, MatchGroup *mg);
int inputsource_open (InputSource *is);
int inputsource_close (InputSource *is);
int inputsource_process (InputSource *is);

#endif

