/****************************************************************/
/* nievent - client to send an event to a netinfoevtd		*/
/****************************************************************/
/* History:							*/
/* (03-10-2005) Orginal creation date. Todd Shadburn		*/
/****************************************************************/

#include <sys/types.h>
#include <sys/stat.h>

#ifndef WIN32
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#else
#include <wtypes.h>
#include <winsock2.h>
#include <io.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <time.h>

#include <comm/comm.h>

#include <netinfo/netinfo.h>
#include <netinfo/event.h>
#include <netinfo/conffile.h>


/****************************************************************/
/* Defines							*/
/****************************************************************/
#define	BUFFER_SIZE		256

#ifndef WIN32
#define NETINFOD_CONFIG_FILE    "/netinfod.conf"
#else
#define NETINFOD_CONFIG_FILE    "\\netinfod.conf"
#endif


/****************************************************************/
/* Global Variables						*/
/****************************************************************/


/****************************************************************/
/* Function Prototypes						*/
/****************************************************************/
void usage(void);

/****************************************************************/
/* Program entry point						*/
/****************************************************************/
int main(int argc, char *argv[])
{
#ifdef WIN32
	WSADATA		wsaData;
#endif
	int		rc;
	int		option;
	u_int32_t	evtflags;
	u_int32_t	evtrc;
	u_int32_t	evtdatalen;
	int		eventrepeats;
        int             bytes;
	char		buffer[BUFFER_SIZE];
	Attribute	*commattribs = NULL;;
        CommModule	*cm = NULL;
        CommChannel     *cc = NULL;
	ConfigFile	*cf = NULL;
	char		*evthostname = NULL;
	char		*evtobject = NULL;
	char		*evtdata = NULL;
	EventData	*event = NULL;
	char		*directive = NULL;
	char		*sinkhost = NULL;

	if(argc < 2) {
		usage();
		exit(0);
	}

	evtflags = 0;
	evtrc = 0;
	evtdatalen = 0;
	eventrepeats = 1;
		
#ifndef WIN32
	while ((option = getopt(argc, argv, "d:f:hH:o:r:R:V")) != EOF) {
		switch(option) {
			case 'd':
				evtdata = strdup(optarg);
				evtdatalen = strlen (evtdata);
				break;
			case 'f':
				evtflags = atoi(optarg);
				break;
			case 'h':
				usage();
				exit(0);
				break;
			case 'H':
				evthostname = strdup(optarg);
				break;
			case 'o':
				evtobject = strdup(optarg);
				break;
			case 'r':
				evtrc = atoi(optarg);
				break;
			case 'R':
				eventrepeats = atoi(optarg);
				break;
			case 'V':
				printf("\nnievent - version %s\n\n",
					VERSION);
				exit(0);
				break;
			default:
				usage();
				exit(1);
				break;
		}
	}
#else
	option = 1;
	while (option < argc) {
		if(argv[option][0] == '-') {
			switch(argv[option][1]) {
			case 'd':
				if((option+1) < argc) {
					option++;
					evtdata = strdup(argv[option]);
					evtdatalen = strlen (evtdata);
				}
				break;
			case 'f':
				if((option+1) < argc) {
					option++;
					evtflags = atoi(argv[option]);
				}
				break;
			case 'h':
				usage();
				exit(0);
				break;
			case 'H':
				if((option+1) < argc) {
					option++;
					evthostname = strdup(argv[option]);
				}
				break;
			case 'o':
				if((option+1) < argc) {
					option++;
					evtobject = strdup(argv[option]);
				}
				break;
			case 'r':
				if((option+1) < argc) {
					option++;
					evtrc = atoi(argv[option]);
				}
				break;
			case 'R':
				if((option+1) < argc) {
					option++;
					eventrepeats = atoi(argv[option]);
				}
				break;
			case 'V':
				printf("\nnievent - version %s\n\n",
					VERSION);
				exit(0);
				break;
			default:
				usage();
				exit(1);
				break;
			}
		} else {
			usage();
			exit(1);
		}
		option++;
	}
#endif

	rc = configfile_open (&cf, SYSCONFDIR NETINFOD_CONFIG_FILE);
	if (rc != NETINFO_RC_SUCCESS) {
		fprintf (stderr, "Failed to open configuration file.\n");
		return 2;
	}
	rc = configfile_search_begin (cf, "EVENTHOST");
	rc = configfile_search_next (cf, buffer, BUFFER_SIZE);
	if (rc != NETINFO_RC_SUCCESS) {
		fprintf (stderr, "Failed to find event sink directive.\n");
		rc = configfile_close (&cf);
		return 2;
	}
	directive = strtok (buffer, " \t");
	sinkhost = strdup (strtok (NULL, " \r\n"));
	rc = configfile_search_end (cf);

	rc = configfile_search_begin (cf, "PUBLICKEY");
	rc = configfile_search_next (cf, buffer, BUFFER_SIZE);
	if (rc != NETINFO_RC_SUCCESS) {
		fprintf (stderr, "Failed to find SSL public key directive.\n");
		rc = configfile_close (&cf);
		return 2;
	}
	directive = strtok (buffer, " \t");
        commattribs = attribute_add (commattribs,
                                "SSLCertificateFile",
                                strtok (NULL, " \r\n"));
	rc = configfile_search_end (cf);

	rc = configfile_search_begin (cf, "PRIVATEKEY");
	rc = configfile_search_next (cf, buffer, BUFFER_SIZE);
	if (rc != NETINFO_RC_SUCCESS) {
		fprintf (stderr, "Failed to find SSL private key directive.\n");
		rc = configfile_close (&cf);
		attribute_freeall (&commattribs);
		if (sinkhost != NULL)
			free (sinkhost);
		if (evtobject != NULL)
			free (evtobject);
		if (evtdata != NULL)
			free (evtdata);
		if (evthostname != NULL)
			free (evthostname);
		return 2;
	}
	directive = strtok (buffer, " \t");
        commattribs = attribute_add (commattribs,
                                "SSLPrivateKeyFile",
                                strtok (NULL, " \r\n"));
	rc = configfile_search_end (cf);
	rc = configfile_close (&cf);

#ifdef WIN32
	WSAStartup(MAKEWORD(2,0), &wsaData);
#endif

	rc = comm_plugin_load (&cm,
			"comm_openssl",
			commattribs);
	if (rc != NETINFO_RC_SUCCESS) {
		fprintf (stderr,
			"Failed to initialize communications (rc=%d).\n",
			rc);
		attribute_freeall (&commattribs);
		if (sinkhost != NULL)
			free (sinkhost);
		if (evtobject != NULL)
			free (evtobject);
		if (evtdata != NULL)
			free (evtdata);
		if (evthostname != NULL)
			free (evthostname);
		return 3;
	}

	rc = comm_connect (cm, &cc, sinkhost, NIEVENT_DEFAULT_PORT);
	if (rc != NETINFO_RC_SUCCESS) {
		fprintf (stderr, "Failed to connect to event sink.\n");
		attribute_freeall (&commattribs);
		if (sinkhost != NULL)
			free (sinkhost);
		if (evtobject != NULL)
			free (evtobject);
		if (evtdata != NULL)
			free (evtdata);
		if (evthostname != NULL)
			free (evthostname);
		rc = comm_plugin_unload (&cm);
		return 3;
	}

        rc = eventdata_new (&event);
	if (rc != NETINFO_RC_SUCCESS) {
		fprintf (stderr, "Failed to allocate memory for event.\n");
		attribute_freeall (&commattribs);
		if (sinkhost != NULL)
			free (sinkhost);
		if (evtobject != NULL)
			free (evtobject);
		if (evtdata != NULL)
			free (evtdata);
		if (evthostname != NULL)
			free (evthostname);
		rc = comm_close (&cc, 0);
		rc = comm_plugin_unload (&cm);
		return 3;
	}
	rc = eventdata_set_all (event,
			evthostname,
			evtflags,
			evtrc,
			evtobject,
			evtdata,
			evtdatalen);

	rc = eventdata_to_networkorder (event);
	rc = eventdata_sizeof (event, &bytes);
	do {
		rc = comm_write (cc, (char *)event, &bytes);
		if (rc != NETINFO_RC_SUCCESS) {
			fprintf (stderr, "Failed to write event data.\n");
			rc = eventdata_free (&event);
			attribute_freeall (&commattribs);
			if (sinkhost != NULL)
				free (sinkhost);
			if (evtobject != NULL)
				free (evtobject);
			if (evtdata != NULL)
				free (evtdata);
			if (evthostname != NULL)
				free (evthostname);
			rc = comm_close (&cc, 0);
			rc = comm_plugin_unload (&cm);
			return 3;
			}
		eventrepeats--;
	} while (eventrepeats > 0);

	rc = comm_close (&cc, 0);
	rc = comm_plugin_unload (&cm);
	rc = eventdata_free (&event);
	attribute_freeall (&commattribs);
	if (sinkhost != NULL)
		free (sinkhost);
	if (evtobject != NULL)
		free (evtobject);
	if (evtdata != NULL)
		free (evtdata);
	if (evthostname != NULL)
		free (evthostname);
#ifdef WIN32
	WSACleanup();
#endif
	return 0;
}


void usage(void)
{
	printf("\nnievent - Send an event to a netinfoevt trap server.\n");
	printf(" Options:\n");
	printf("    -d \"data\"     Specifies the event data text.\n");
	printf("    -f flags      Specifies the event flags.\n");
	printf("    -h            Show the command usage information.\n");
	printf("    -H hostname   Specifies an alternate hostname to use for the event.\n");
	printf("    -o \"object\"   Specifies the event object text.\n");
	printf("    -r rc         Specifies the event return code.\n");
	printf("    -R count      Specifies the number of times to repeat the event.\n");
	printf("    -V            Show version information for nievent.\n\n");
	return;
}


