
#ifndef WIN32
#include <dlfcn.h>
#else
#include <windows.h>
#include <winsock2.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <stdarg.h>
#include <netinfo/netinfo.h>
#include <netinfo/nmodule_p.h>
#include <netinfo/event.h>

#ifdef WIN32
#define RTLD_NOW		1
#define	dlopen(x,y)		LoadLibrary (x)
#define dlsym(x,y)		GetProcAddress (x, y)
#define dlclose(x)		FreeLibrary (x)
#endif

/* Functions for netinfod use */
int     netinfo_vector_initialize (ModuleVector *mv)
{
	if (mv == NULL)
		return NETINFO_RC_INVALIDARG;
	memset (mv, 0, sizeof (ModuleVector));
	mv->version = MODULE_VECTOR_VERSION;
	return NETINFO_RC_SUCCESS;
}


int     netinfo_vector_set_log_entry (ModuleVector *mv, void *callback)
{
	if (mv == NULL)
		return NETINFO_RC_INVALIDARG;
	if (mv->version >= MODULE_VECTOR_VERSION_1)
		mv->log_entry = callback;
	return NETINFO_RC_SUCCESS;
}


int     netinfo_vector_set_emit_event (ModuleVector *mv, void *callback)
{
	if (mv == NULL)
		return NETINFO_RC_INVALIDARG;
	if (mv->version >= MODULE_VECTOR_VERSION_1)
		mv->emit_event = callback;
	return NETINFO_RC_SUCCESS;
}


int     netinfo_module_load (NetinfoModule **nmp, char *path, ModuleVector *mv)
{
	int rc;
	NetinfoModule *nm = NULL;
	if (nmp == NULL || path == NULL || mv == NULL)
		return NETINFO_RC_INVALIDARG;
	*nmp = NULL;
	nm = calloc (1, sizeof (NetinfoModule));
	if (nm == NULL)
		return ENOMEM;
	nm->moduleinfo.vector = mv;
	nm->module = dlopen (path, RTLD_NOW);
	if (nm->module == NULL) {
		free (nm);
		return NETINFO_RC_LOADERROR;
	}
	nm->moduleinit = dlsym (nm->module, "netinfo_module_init");
	if (nm->moduleinit == NULL) {
		dlclose (nm->module);
		free (nm);
		return NETINFO_RC_MODULEERROR;
	}
	nm->moduleclose = dlsym (nm->module, "netinfo_module_close");
	if (nm->moduleclose == NULL) {
		dlclose (nm->module);
		free (nm);
		return NETINFO_RC_MODULEERROR;
	}
	nm->modulecallmethod = dlsym (nm->module, "netinfo_module_execute_method");
	rc = (*nm->moduleinit) (&nm->moduleinfo);
	if (rc != NETINFO_RC_SUCCESS) {
		dlclose (nm->module);
		free (nm);
		return NETINFO_RC_MODULEERROR;
	}
	*nmp = nm;
	return NETINFO_RC_SUCCESS;
}



const char*   netinfo_module_get_name (NetinfoModule *nm)
{
	if (nm == NULL)
		return NULL;
	return nm->moduleinfo.name;
	return NULL;
}


int     netinfo_module_call_method (NetinfoModule *nm,
                                        char *method,
                                        NetinfoHeader *nhresponse,
                                        char **responsedata,
                                        u_int32_t *responsedatalen)
{
	int rc;
	if (nm == NULL || method == NULL ||
		nhresponse == NULL)
		return NETINFO_RC_INVALIDARG;
	if (nm->modulecallmethod == NULL)
		return NETINFO_RC_NOTIMPLEMENTED;
	rc = (*nm->modulecallmethod)(method,
					nhresponse,
					responsedata,
					responsedatalen);
	return rc;
}


int     netinfo_module_unload (NetinfoModule **nmp)
{
	int rc;
	NetinfoModule *nm = NULL;
	if (nmp == NULL || *nmp == NULL)
		return NETINFO_RC_INVALIDARG;
	nm = *nmp;
	if (nm->moduleclose != NULL)
		rc = (*nm->moduleclose)(&nm->moduleinfo);
	if (nm->module != NULL) {
		rc = dlclose (nm->module);
		nm->module = NULL;
	}
	memset (nm, 0, sizeof (NetinfoModule));
	free (nm);
	*nmp = NULL;
	return NETINFO_RC_SUCCESS;
}


/* Functions for plugin(s) use */
int     vector_log_entry (ModuleInfo *mi, char *format, ...)
{
	va_list args;
	int rc;
	if (mi == NULL || format == NULL)
		return NETINFO_RC_INVALIDARG;
	va_start (args, format);
	rc = (*mi->vector->log_entry) (mi, format, args);
	va_end (args);
	return rc;
}


int     vector_emit_event (ModuleInfo *mi, EventData *ed)
{
	int rc;
	if (mi == NULL || ed == NULL)
		return NETINFO_RC_INVALIDARG;
	rc = (*mi->vector->emit_event) (mi, ed);
	return rc;
}







